	SUBROUTINE QAG(F,A,B,EPSABS,EPSREL,KEY,RESULT,ABSERR,NEVAL,IER)
C
C .....................................................................
C
C 1.	QAG
C	COMPUTATION OF A DEFINITE INTEGRAL
C	   STANDARD FORTRAN SUBROUTINE
C
C 2.	PURPOSE
C	   THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C	   DEFINITE INTEGRAL I = INTEGRAL OF   F   OVER  (A,B),
C	   HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C	   ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.	CALLING SEQUENCE
C	   CALL QAG(F,A,B,EPSABS,EPSREL,KEY,RESULT,ABSERR,NEVAL,IER)
C
C	PARAMETERS
C	 ON ENTRY
C	    F 	   - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR N NEEDS TO BE
C     		     DECLARED  E X T E R N A L  IN THE DRIVER PROGRAM.
C
C	    A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C	    B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C	    EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF  EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C	    KEY    - INTEGER
C                     KEY FOR CHOICE OF LOCAL INTEGRATION RULE
C                     A GAUSS-KRONROD PAIR IS USED WITH
C                     7 - 15 POINTS IF KEY.LT.2,
C                    10 - 21 POINTS IF KEY = 2,
C                    15 - 31 POINTS IF KEY = 3,
C                    20 - 41 POINTS IF KEY = 4,
C                    25 - 51 POINTS IF KEY = 5,
C                    30 - 61 POINTS IF KEY.GT.5.
C
C       ON RETURN
C	    RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR RESULT AND ERROR ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE DATA VALUE
C                            OF LIMIT IN QAG (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENTS INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT IT
C                            IS ADVISED TO ANALYSE THE INTEGRAND IN
C                            ORDER TO DETERMINE THE INTEGRATION
C                            DIFFICULTIES. IF THE POSITION OF A LOCAL
C                            DIFFICULTY CAN BE DETERMINED (E. G.
C                            SINGULARITY, DISCONTINUITY WITHIN THE
C                            INTERVAL) ONE WILL PROBABLY GAIN FROM
C                            SPLITTING UP THE INTERVAL AT THIS POINT
C                            AND CALLING THE INTEGRATOR ON THE
C                            SUBRANGES. IF POSSIBLE, AN APPROPRIATE
C                            SPECIAL-PURPOSE INTEGRATOR SHOULD BE USED
C                            WHICH IS DESIGNED FOR HANDLING THE TYPE OF
C                            DIFFICULTY INVOLVED.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR, NEVAL ARE SET TO ZERO.
C
C 4.	SUBROUTINES OR FUNCTIONS NEEDED
C             - QAGE
C             - QK15, QK21,QK31, QK41, QK51, AND QK61
C             - QSORT
C             - F (USER-PROVIDED FUNCTION)
C             - QMACO
C             - FORTRAN ABS, AMAX1, AMIN1
C
C....................................................................
C
      REAL*8 A,ABSERR,ALIST,B,BLIST,ELIST,EPSABS,EPSREL,F,RESULT,RLIST
      INTEGER IER,IORD,KEY,LAST,LIMIT,NEVAL
C
      DIMENSION ALIST(500),BLIST(500),ELIST(500),IORD(500),RLIST(500)
C
	EXTERNAL F
C
C	LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN
C	THE SUBDIVISION PROCESS OF QAGE. TAKE CARE THAT LIMIT.GE.1.
C
	DATA LIMIT/500/
C
C	KEY DETERMINES THE QUADRATURE FORMULAE TO BE APPLIED
C	BY DQAGE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QAGE(F,A,B,EPSABS,EPSREL,KEY,LIMIT,RESULT,ABSERR,NEVAL,IER,
     *  ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
	RETURN
        END
