$debug
      SUBROUTINE QAGE(F,A,B,EPSABS,EPSREL,KEY,LIMIT,RESULT,ABSERR,NEVAL,
     *  IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
C.....................................................................
C
C 1.	QAGE
C	COMPUTATION OF A DEFINITE INTEGRAL
C	   STANDARD FORTRAN SUBROUTINE
C
C 2.	PURPOSE
C          THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C	DEFINITE INTEGRAL   I = INTEGRAL OF F OVER (A,B),
C	HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C	ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.	CALLING SEQUENCE
C	   CALL QAGE(F,A,B,EPSABS,EPSREL,KEY,LIMIT,RESULT,ABSER,NEVAL,
C                    IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
C	PARAMETERS
C        ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF  EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C           KEY    - INTEGER
C                    KEY FOR CHOICE OF LOCAL INTEGRATION RULE
C                    A GAUSS-KRONROD PAIR IS USED WITH
C                         7 - 15 POINTS IF KEY.LT.2,
C                        10 - 21 POINTS IF KEY = 2,
C                        15 - 31 POINTS IF KEY = 3,
C                        20 - 41 POINTS IF KEY = 4,
C                        25 - 51 POINTS IF KEY = 5,
C                        30 - 61 POINTS IF KEY.GT.5.
C
C	    LIMIT  - INTEGER
C                    GIVES AN UPPERBOUND ON THE NUMBER OF SUBINTERVALS
C                    IN THE PARTITION OF (A,B), LIMIT.GE.1.
C
C	ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR RESULT AND ERRORS ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE VALUE OF
C                            LIMIT. HOWEVER, IF THIS YIELDS NO
C                            IMPROVEMENT IT IS ADVISED TO ANALYSE THE
C                            INTEGRAND IN ORDER TO DETERMINE THE
C                            INTEGRATION DIFFICULTIES. IF THE POSITION
C                            OF A LOCAL DIFFICULTY CAN BE DETERMINED
C                            (E.G. SINGULARITY, DISCONTINUITY WITHIN
C                            THE INTERVAL) ONE WILL PROBABLY GAIN FROM
C                            SPLITTING UP THE INTERVAL AT THIS POINT
C                            AND CALLING THE INTEGRATOR ON THE
C                            SUBRANGES. IF POSSIBLE, AN APPROPRIATE
C                            SPECIAL-PURPOSE INTEGRATOR SHOULD BE USED
C                            WHICH IS DESIGNED FOR HANDLING THE TYPE OF
C                            DIFFICULTY INVOLVED.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INAVLID, BECAUSE
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR, NEVAL, LAST, RLIST(1) ,
C                            ELIST(1) AND IORD(1) ARE SET TO ZERO.
C                            ALIST(11) AND BLIST(1) ARE SET TO A AND B
C                            RESPECTIVELY.
C
C           ALIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE LEFT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B)
C
C           BLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE RIGHT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B).
C
C           RLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE INTEGRAL
C                    APPROXIMATIONS ON THE SUBINTERVALS
C
C           ELIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE MODULI OF THE
C                    ABSOLUTE ERROR ESTIMATES ON THE SUBINTERVALS
C
C           IORD   - INTEGER
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST K
C                    ELEMENTS OF WHICH ARE POINTERS TO THE ERROR
C                    ESTIMATES OVER THE SUBINTERVALS, SUCH THAT
C                    ELIST(IORD(1)),..., ELIST(IORD(K)) FORM A
C                    DECREASING SEQUENCE, WITH K = LAST
C                    IF LAST.LE.(LIMIT/2+2), AND K = LIMIT+1-LAST
C                    OTHERWISE
C
C           LAST   - INTEGER
C                    NUMBER OF SUBINTERVALS ACTUALLY PRODUCED IN THE
C                    SUBDIVISION PROCESS.
C
C 4.	SUBROUTINES OR FUNCTIONS NEEDED
C             - QK15, QK21, QK31, QK41, QK51, QK61
C             - QSORT
C             - F (USER-PROVIDED FUNCTION)
C             - QMACO
C             - FORTRAN ABS, AMAX1, AMIN1
C
C....................................................................
C
      REAL*8
     *A,ABSERR,ALIST,AREA,AREA1,AREA12,AREA2,A1,A2,B,BLIST,B1,B2,C,
     *  DEFABS,DEFAB1,DEFAB2,ELIST,EPMACH,EPSABS,EPSREL,ERRBND,ERRMAX,
     *  ERROR1,ERROR2,ERRO12,ERRSUM,F,OFLOW,RESABS,RESULT,RLIST,UFLOW
      INTEGER IER,IORD,IROFF1,IROFF2,K,KEY,KEYF,LAST,LIMIT,MAXERR,NEVAL,
     *  NRMAX
C
      DIMENSION ALIST(LIMIT),BLIST(LIMIT),ELIST(LIMIT),IORD(LIMIT),
     *  RLIST(LIMIT)
C
      EXTERNAL F
C
C            LIST OF MAJOR VARIABLES
C            -----------------------
C
C           ALIST     - LIST OF LEFT END POINTS OF ALL SUBINTERVALS
C                       CONSIDERED UP TO NOW
C           BLIST     - LIST OF RIGHT END POINTS OF ALL SUBINTERVALS
C                       CONSIDERED UP TO NOW
C           RLIST(I)  - APPROXIMATION TO THE INTEGRAL OVER
C                       (ALIST(I),BLIST(I))
C           ELIST(I)  - ERROR ESTIMATE APPLYING TO RLIST(I)
C           MAXERR    - POINTER TO THE INTERVAL WITH LARGEST ERROR
C                       ESTIMATE
C           ERRMAX    - ELIST(MAXERR)
C           AREA      - SUM OF THE INTEGRALS OVER THE SUBINTERVALS
C           ERRSUM    - SUM OF THE ERRORS OVER THE SUBINTERVAL
C           ERRBND    - REQUESTED ACCURACY MAX(EPSABS,EPSREL*
C                       ABS(RESULT))
C           *****1    - VARIABLE FOR THE LEFT SUBINTERVAL
C           *****2    - VARIABLE FOR THE RIGHT SUBINTERVAL
C           LAST      - INDEX FOR SUBDIVISION
C
C
C   	    MACHINE DEPENDENT CONSTANTS
C           ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
	CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C	TEST ON VALIDITY OF PARAMETERS
C       ------------------------------
C
	IER = 0
        NEVAL = 0
        LAST = 0
        RESULT = 0.0D+00
        ABSERR = 0.0D+00
        ALIST(1) = A
        BLIST(1) = B
        RLIST(1) = 0.0D+00
        ELIST(1) = 0.0D+00
        IORD(1) = 0
        IF(EPSABS.LT.0.D0.AND.EPSREL.LT.0.D0) IERR = 6
        IF(IER.EQ.6) GO TO 999
C
C	FIRST APPROXIMATION TO THE INTEGRAL
C       -----------------------------------
C
	KEYF = KEY
        IF(KEY.LE.0) KEYF = 1
        IF(KEY.GE.7) KEYF = 6
        C = KEYF
        NEVAL = 0
        IF(KEYF.EQ.1) CALL QK15(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        IF(KEYF.EQ.2) CALL QK21(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        IF(KEYF.EQ.3) CALL QK31(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        IF(KEYF.EQ.4) CALL QK41(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        IF(KEYF.EQ.5) CALL QK51(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        IF(KEYF.EQ.6) CALL QK61(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
        LAST = 1
        RLIST(1) = RESULT
        ELIST(1) = ABSERR
        IORD(1) = 1
C
C	TEST OF ACCURACY
C
       ERRBND = AMAX1(EPSABS,EPSREL*ABS(RESULT))
       IF(ABSERR.LE.5.D+01*EPMACH*DEFABS.AND.ABSERR.GT.ERRBND) IER = 2
       IF(LIMIT.EQ.1) IER = 1
       IF(IR.NE.0.OR.(ABSERR.LE.ERRBND.AND.ABSERR.NE.RESABS)
     *   .OR.ABSERR.EQ.0.D+00) GO TO 60
C
C	INITIALIZATION
C       --------------
C
C
	ERRMAX = ABSERR
        MAXERR = 1
        AREA = RESULT
        ERRSUM = ABSERR
        NRMAX = 1
        IROFF1 = 0
        IROFF2 = 0
C
C	MAIN DO-LOOP
C       ------------
C
	DO 30 LAST = 2,LIMIT
C
C	BISECT THE SUBINTERVAL WITH THE LARGEST ERROR ESTIMATE
C
	A1 = ALIST(MAXERR)
        B1 = 5.D-01*(ALIST(MAXERR)+BLIST(MAXERR))
        A2 = B1
        B2 = BLIST(MAXERR)
        IF(KEYF.EQ.1) CALL QK15(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.2) CALL QK21(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.3) CALL QK31(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.4) CALL QK41(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.5) CALL QK51(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.6) CALL QK61(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        IF(KEYF.EQ.1) CALL QK15(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
        IF(KEYF.EQ.2) CALL QK21(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
        IF(KEYF.EQ.3) CALL QK31(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
        IF(KEYF.EQ.4) CALL QK41(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
        IF(KEYF.EQ.5) CALL QK51(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
        IF(KEYF.EQ.6) CALL QK61(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
C
C	  IMPROVE PREVIOUS APPROXIMATIONS TO INTEGRAL AND ERROR AND
C	  TEST FOR ACCURACY
C
	NEVAL = NEVAL+1
        AREA12 = AREA1+AREA2
        ERRO12 = ERROR1+ERROR2
        ERRSUM = ERRSUM+ERRO12-ERRMAX
        AREA = AREA+AREA12-RLIST(MAXERR)
        IF(DEFAB1.EQ.ERROR1.OR.DEFAB2.EQ.ERROR2) GO TO 5
        IF(ABS(RLIST(MAXERR)-AREA12).LE.1.0D-05*DABS(AREA12)
     *    .AND.ERRO12.GE.9.9D-01*ERRMAX) IROFF1 = IROFF1+1
        IF(LAST.GT.10.AND.ERRO12.GT.ERRMAX) IROFF2 = IROFF2+1
    5	RLIST(MAXERR) = AREA1
    	RLIST(LAST) = AREA2
        ERRBND = AMAX1(EPSABS,EPSREL*ABS(AREA))
        IF(ERRSUM.LE.ERRBND) GO TO 8
C
C	TEST FOR ROUNDOFF ERROR AND EVENTUALLY SET ERROR FLAG
C
	IF(IROFF1.GE.6.OR.IROFF2.GE.20) IER = 2
C
C    SET ERROR FLAG IN THE CASE THAT THE NUMBER OF SUBINTERVALS
C    EQUALS LIMIT
C
	IF(LAST.EQ.LIMIT) IER = 1
C
C     SET ERROR FLAG IN THE CASE OF BAD INTEGRAND BEHAVIOUR
C     AT A POINT OF THE INTEGRATION RANGE.
C
      IF(DMAX1(DABS(A1),DABS(B2)).LE.(1.D0+C*1.0D+03*
     * EPMACH)*(DABS(A2)+1.0D+04*UFLOW)) IER = 3
C
C    APPEND THE NEWLY-CREATED INTERVALS TO THE LIST
C
    8 IF(ERROR2.GT.ERROR1) GO TO 10
    	ALIST(LAST) = A2
        BLIST(MAXERR) = B1
        BLIST(LAST) = B2
        ELIST(MAXERR) = ERROR1
        ELIST(LAST) = ERROR2
        GO TO 20
  10	ALIST(MAXERR) = A2
        ALIST(LAST) = A1
        BLIST(LAST) = B1
        RLIST(MAXERR) = AREA2
        RLIST(LAST) = AREA1
        ELIST(MAXERR) = ERROR2
        ELIST(LAST) = ERROR1
C
C	CALL SUBROUTINE QSQRT TO MAINTAIN THE DESCENDING ORDERING
C       IN THE LIST OF ERROR ESTIMATES AND SELECT THE SUBINTERVAL
C	WITH THE LARGEST ERROR ESTIMATE (TO BE BISECTED NEXT).
C
   20	CALL QSORT(LIMIT,LAST,MAXERR,ERRMAX,ELIST,IORD,NRMAX)
C***JUMP OUT OF DO-LOOP
	IF(IER.NE.0.OR.ERRSUM.LE.ERRBND) GO TO 40
   30	CONTINUE
C
C	COMPUTE FINAL RESULT.
C       ---------------------
C
   40	RESULT = 0.D0
   	DO 50 K=1,LAST
          RESULT = RESULT + RLIST(K)
   50	CONTINUE
   	ABSERR = ERRSUM
   60	IF(KEYF.NE.1) NEVAL = (10*KEYF+1)*(2*NEVAL+1)
    	IF(KEYF.EQ.1) NEVAL = 30*NEVAL+15
  999	RETURN
  	END
