      SUBROUTINE QAGP(F,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER)
C
C .....................................................................
C
C 1.     QAGP
C        COMPUTATION OF A DEFINITE INTEGRAL
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C           DEFINITE INTEGRAL  I= INTEGRAL OF F OVER (A,B),
C           HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C           BREAK POINTS OF THE INTEGRATION INTERVAL, WHERE LOCAL
C           DIFFICULTIES OF THE INTEGRAND MAY OCCUR (E.G. SINGULARITIES,
C           DISCONTINUITIES), ARE PROVIDED BY THE USER.
C
C 3.     CALLING SEQUENCE
C           CALL QAGP(F,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
C                     NEVAL,IER)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINIG THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           NPTS2  - INTEGER
C                    NUMBER EQUAL TO TWO MORE THAN THE NUMBER OF
C                    USER-SUPPLIED INTERIOR BREAK POINTS WITHIN THE
C                    INTEGRATION RANGE, NPTS2.GE.2.
C                    IF NPTS2.LT.2, THE ROUTINE WILL END WITH IER = 6.
C
C           POINTS - REAL
C                    VECTOR OF DIMENSION NPTS2, THE FIRST (NPTS2-2)
C                    ELEMENTS OF WHICH ARE THE USER PROVIDED INTERIOR
C                    BREAK POINTS. IF THESE POINTS DO NOT CONSTITUTE
C                    AN ASCENDING SEQUENCE THERE WILL BE AN AUTOMATIC
C                    SORTING.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR THE INTEGRAL AND ERROR
C                            ARE LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE DATA VALUE
C                            OF LIMIT IN QAGP (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENTS INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT IT
C                            IS ADVISED TO ANALYZE THE INTEGRAND, IN
C                            ORDER TO DETERMINE THE INTEGRATION
C                            DIFFICULTIES. IF THE POSITION OF A LOCAL
C                            DIFFICULTY CAN BE DETERMINED (I.E.
C                            SINGULARITIY, DISCONTINUITY WITHIN THE
C                            INTERVAL), IT SHOULD BE SUPPLIED TO THE
C                            ROUTINE AS AN ELEMENT OF THE VECTOR
C                            POINTS. IF NECESSEARY, AN APPROPRIATE
C                            SPECIAL PURPOSE INTEGRATOR MUST BE USED,
C                            WHICH IS DESIGNED FOR HANDLING THE TYPE
C                            OF DIFFICULTY INVOLVED.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                            THE ERROR MAY BE UNDER-ESTIMATED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 4 THE ALGORITHM DOES NOT CONVERGE. ROUNDOFF
C                            ERROR IS DETECTED IN THE EXTRAPOLATION
C                            TABLE. IT IS ASSUMED THAT THE REQUESTED
C                            TOLERANCE CANNOT BE ACHIEVED, AND THAT THE
C                            RETURNED RESULT IS THE BEST WHICH
C                            CAN BE OBTAINED.
C                        = 5 THE INTEGRAL IS PROBABLY DIVERGENT, OR
C                            SLOWLY CONVERGENT. IT MUST BE NOTED THAT
C                            DIVERGENCE CAN OCCUR WITH ANY OTHER VALUE
C                            OF IER.GT.0.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            NPTS2.LT.2 OR
C                            BREAK POINTS ARE SPECIFIED OUTSIDE
C                            THE INTEGRATION RANGE OR
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            OR LIMIT.LT.NPTS2.
C                            RESULT, ABSERR, NEVAL ARE SET TO ZERO.
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C                 - QK21
C                 - QSORT
C                 - QEXT
C                 - F (USER-PROVIDED FUNCTION)
C                 - QMACO
C                 - FORTRAN DABS, DMAX1, DMIN1
C
C .....................................................................
C
      REAL*8 A,ABSEPS,ABSERR,ALIST,AREA,AREA1,AREA12,AREA2,A1,A2,B,
     *  BLIST,
     *  B1,B2,CORREC,DEFABS,DEFAB1,DEFAB2,DRES,ELIST,EPMACH,EPSABS,
     *  EPSREL,ERLARG,ERLAST,ERRBND,ERRMAX,ERROR1,ERRO12,ERROR2,ERRSUM,
     *  ERTEST,F,OFLOW,POINTS,PTS,RESA,RESABS,RESEPS,RESULT,RES3LA,
     *  RLIST,RLIST2,SIGN,TEMP,UFLOW
      INTEGER I,ID,IER,IERRO,IND1,IND2,IORD,IP1,IROFF1,IROFF2,IROFF3,J,
     *  JLOW,JUPBND,K,KSGN,KTMIN,LAST,LEVCUR,LEVEL,LEVMAX,LIMIT,MAXERR,
     *  NDIN,NEVAL,NINT,NINTP1,NPTS,NPTS2,NRES,NRMAX,NUMRL2
      LOGICAL EXTRAP,NOEXT
C
      DIMENSION ALIST(500),BLIST(500),ELIST(500),IORD(500),LEVEL(500),
     *  NDIN(40),POINTS(40),PTS(40),RES3LA(3),RLIST(500),RLIST2(52)
C
C           THE DIMENSION OF RLIST2 IS DETERMINED BY THE VALUE OF
C           LIMEXP IN SUBROUTINE EPSALG (RLIST2 SHOULD BE OF DIMENSION
C           (LIMEXP+2) AT LEAST.
C
      EXTERNAL F
C
C           LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN THE
C           SUBDIVISION PROCES OF QAGP. TAKE CARE THAT LIMIT.GE.2.
C
      DATA LIMIT/500/
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           ALIST    - LIST OF LEFT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           BLIST    - LIST OF RIGHT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           RLIST(I) - APROXIMATION TO THE INTEGRAL OVER
C                      (ALIST(I),BLIST(I))
C           RLIST2   - ARRAY OF DIMENSION AT LEAST (LIMEXP+2),
C                      CONTAINING THE PART OF THE EPSILON TABLE WHICH
C                      IS STILL NEEDED FOR FURTHER COMPUTATIONS
C           ELIST(I) - ERROR ESTIMATE APPLYING TO RLIST(I)
C           MAXERR   - POINTER TO THE INTERVAL WITH LARGEST ERROR
C                      ESTIMATE
C           ERRMAX   - ELIST(MAXERR)
C           ERLAST   - ERROR ON THE INTERVAL CURRENTLY SUBDIVIDED
C                      (BEFORE THAT SUBDIVISION HAS TAKEN PLACE)
C           AREA     - SUM OF THE INTEGRALS OVER THE SUBINTERVALS
C           ERRSUM   - SUM OF THE ERRORS OVER THE SUBINTERVALS
C           ERRBND   - REQUESTED ACCURACY MAX(EPSABS,EPSREL*
C                      ABS(RESULT))
C           *****1   - VARIABLE FOR THE LEFT SUBINTERVAL
C           *****2   - VARIABLE FOR THE RIGHT SUBINTERVAL
C           LAST     - INDEX FOR SUBDIVISION
C           NRES     - NUMBER OF CALLS TO THE EXTRAPOLATION ROUTINE
C           NUMRL2   - NUMBER OF ELEMENTS IN RLIST2. IF AN APPROPRIATE
C                      APPROXIMATION TO THE COMPOUNDED INTEGRAL HAS
C                      OBTAINED, IT IS PUT IN RLIST2(NUMRL2) AFTER
C                      NUMRL2 HAS BEEN INCREASED BY ONE.
C           ERLARG   - SUM OF THE ERRORS OVER THE INTERVALS LARGER
C                      THAN THE SMALLEST INTERVAL CONSIDERED UP TO NOW
C           EXTRAP   - LOGICAL VARIABLE DENOTING THAT THE ROUTINE IS
C                      ATTEMPTING TO PERFORM EXTRAPOLATION, I.E. BEFORE
C                      SUBDIVIDING THE SMALLEST INTERVAL WE TRY TO
C                      DECREASE THE VALUE OF ERLARG
C           NOEXT    - LOGICAL VARIABLE DENOTING THAT EXTRAPOLATION
C                      IS NO LONGER ALLOWED (TRUE VALUE)
C
C           MACHINE DEPENDENT CONSTANTS
C           ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST POSITIVE MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C         TEST ON VALIDITY OF PARAMETERS
C         ------------------------------
C
      IER = 0
      NEVAL = 0
      LAST = 0
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      ALIST(1) = A
      BLIST(1) = B
      RLIST(1) = 0.0D+00
      ELIST(1) = 0.0D+00
      IORD(1) = 0
      LEVEL(1) = 0
      NPTS = NPTS2-2
      IF(NPTS2.LT.2.OR.LIMIT.LE.NPTS.OR.(EPSABS.LT.0.0D+00.AND.
     *  EPSREL.LT.0.0D+00)) IER = 6
      IF(IER.EQ.6) GO TO 999
C
C           IF ANY BREAK POINTS ARE PROVIDED, SORT THEM INTO AN
C           ASCENDING SEQUENCE.
C
      SIGN = 1.0D+00
      IF(A.GT.B) SIGN = -1.0D+00
      PTS(1) = DMIN1(A,B)
      IF(NPTS.EQ.0) GO TO 15
      DO 10 I = 1,NPTS
        PTS(I+1) = POINTS(I)
   10 CONTINUE
   15 PTS(NPTS+2) = DMAX1(A,B)
      NINT = NPTS+1
      A1 = PTS(1)
      IF(NPTS.EQ.0) GO TO 40
      NINTP1 = NINT+1
      DO 20 I = 1,NINT
        IP1 = I+1
        DO 20 J = IP1,NINTP1
          IF(PTS(I).LE.PTS(J)) GO TO 20
          TEMP = PTS(I)
          PTS(I) = PTS(J)
          PTS(J) = TEMP
   20 CONTINUE
      IF(PTS(1).NE.DMIN1(A,B).OR.PTS(NINTP1).NE.DMAX1(A,B)) IER = 6
      IF(IER.EQ.6) GO TO 999
C
C           COMPUTE FIRST INTEGRAL AND ERROR APPROXIMATIONS.
C           ------------------------------------------------
C
   40 RESABS = 0.0D+00
      DO 50 I = 1,NINT
        B1 = PTS(I+1)
        CALL QK21(F,A1,B1,AREA1,ERROR1,DEFABS,RESA)
        ABSERR = ABSERR+ERROR1
        RESULT = RESULT+AREA1
        NDIN(I) = 0
        IF(ERROR1.EQ.RESA.AND.ERROR1.NE.0.0E+00) NDIN(I) = 1
        RESABS = RESABS+DEFABS
        LEVEL(I) = 0
        ELIST(I) = ERROR1
        ALIST(I) = A1
        BLIST(I) = B1
        RLIST(I) = AREA1
        IORD(I) = I
        A1 = B1
   50 CONTINUE
      ERRSUM = 0.0D+00
      DO 55 I = 1,NINT
        IF(NDIN(I).EQ.1) ELIST(I) = ABSERR
        ERRSUM = ERRSUM+ELIST(I)
   55 CONTINUE
C
C           TEST ON ACCURACY.
C
      LAST = NINT
      NEVAL = 21*NINT
      DRES = DABS(RESULT)
      ERRBND = DMAX1(EPSABS,EPSREL*DRES)
      IF(ABSERR.LE.1.0D+02*EPMACH*RESABS.AND.ABSERR.GT.ERRBND) IER = 2
      IF(NINT.EQ.1) GO TO 80
      DO 70 I = 1,NPTS
        JLOW = I+1
        IND1 = IORD(I)
        DO 60 J = JLOW,NINT
          IND2 = IORD(J)
          IF(ELIST(IND1).GT.ELIST(IND2)) GO TO 60
          IND1 = IND2
          K = J
   60 CONTINUE
      IF(IND1.EQ.IORD(I)) GO TO 70
        IORD(K) = IORD(I)
        IORD(I) = IND1
   70 CONTINUE
      IF(LIMIT.LT.NPTS2) IER = 1
   80 IF(IER.NE.0.OR.ABSERR.LE.ERRBND) GO TO 999
C
C           INITIALIZATION
C           --------------
C
      RLIST2(1) = RESULT
      MAXERR = IORD(1)
      ERRMAX = ELIST(MAXERR)
      AREA = RESULT
      NRMAX = 1
      NRES = 0
      NUMRL2 = 1
      KTMIN = 0
      EXTRAP = .FALSE.
      NOEXT = .FALSE.
      ERLARG = ERRSUM
      ERTEST = ERRBND
      LEVMAX = 1
      IROFF1 = 0
      IROFF2 = 0
      IROFF3 = 0
      IERRO = 0
      ABSERR = OFLOW
      KSGN = -1
      IF(DRES.GE.(1.0D+00-5.0D+01*EPMACH)*RESABS) KSGN = 1
C
C           MAIN DO-LOOP
C           ------------
C
      DO 160 LAST = NPTS2,LIMIT
C
C           BISECT THE SUBINTERVAL WITH THE NRMAX-TH LARGEST ERROR
C           ESTIMATE.
C
      LEVCUR = LEVEL(MAXERR)+1
      A1 = ALIST(MAXERR)
      B1 = 5.0D-01*(ALIST(MAXERR)+BLIST(MAXERR))
      A2 = B1
      B2 = BLIST(MAXERR)
      ERLAST = ERRMAX
      CALL QK21(F,A1,B1,AREA1,ERROR1,DEFAB1,RESA)
      CALL QK21(F,A2,B2,AREA2,ERROR2,DEFAB2,RESA)
C
C           IMPROVE PREVIOUS APPROXIMATIONS TO INTEGRAL AND ERROR
C           AND TEST FOR ACCURACY.
C
      NEVAL = NEVAL+42
      AREA12 = AREA1+AREA2
      ERRO12 = ERROR1+ERROR2
      ERRSUM = ERRSUM+ERRO12-ERRMAX
      AREA = AREA+AREA12-RLIST(MAXERR)
      IF(DEFAB1.EQ.ERROR1.OR.DEFAB2.EQ.ERROR2) GO TO 95
      IF(DABS(RLIST(MAXERR)-AREA12).GT.1.0E-05*DABS(AREA12)
     *  .OR.ERRO12.LT.9.9D-01*ERRMAX) GO TO 90
      IF(EXTRAP) IROFF2 = IROFF2+1
      IF(.NOT.EXTRAP) IROFF1 = IROFF1+1
   90 IF(LAST.GT.10.AND.ERRO12.GT.ERRMAX) IROFF3 = IROFF3+1
   95 LEVEL(MAXERR) = LEVCUR
      LEVEL(LAST) = LEVCUR
      RLIST(MAXERR) = AREA1
      RLIST(LAST) = AREA2
      ERRBND = DMAX1(EPSABS,EPSREL*DABS(AREA))
C
C           TEST FOR ROUNDOFF ERROR AND EVENTUALLY SET ERROR FLAG.
C
        IF(IROFF1+IROFF2.GE.10.OR.IROFF3.GE.20) IER = 2
        IF(IROFF2.GE.5) IERRO = 3
C
C           SET ERROR FLAG IN THE CASE THAT THE NUMBER OF SUBINTERVALS
C           EQUALS LIMIT.
C
        IF(LAST.EQ.LIMIT) IER = 1
C
C           SET EROR FLAG IN THE CASE OF BAD INTEGRAND BEHAVIOUR
C           AT A POINT OF THE INTEGRATION RANGE
C
        IF(DMAX1(DABS(A1),DABS(B2)).LE.(1.0D+00+1.0D+03*EPMACH)*
     *  (DABS(A2)+1.0D+03*UFLOW)) IER = 4
C
C           APPEND THE NEWLY-CREATED INTERVALS TO THE LIST.
C
        IF(ERROR2.GT.ERROR1) GO TO 100
        ALIST(LAST) = A2
        BLIST(MAXERR) = B1
        BLIST(LAST) = B2
        ELIST(MAXERR) = ERROR1
        ELIST(LAST) = ERROR2
        GO TO 110
  100   ALIST(MAXERR) = A2
        ALIST(LAST) = A1
        BLIST(LAST) = B1
        RLIST(MAXERR) = AREA2
        RLIST(LAST) = AREA1
        ELIST(MAXERR) = ERROR2
        ELIST(LAST) = ERROR1
C
C           CALL SUBROUTINE QSORT TO MAINTAIN THE DESCENDING ORDERING
C           IN THE LIST OF ERROR ESTIMATES AND SELECT THE SUBINTERVAL
C           WITH NRMAX-TH LARGEST ERROR ESTIMATE (TO BE BISECTED NEXT).
C
  110   CALL QSORT(LIMIT,LAST,MAXERR,ERRMAX,ELIST,IORD,NRMAX)
C***JUMP OUT OF DO-LOOP
        IF(ERRSUM.LE.ERRBND) GO TO 190
C***JUMP OUT OF DO-LOOP
        IF(IER.NE.0) GO TO 170
        IF(NOEXT) GO TO 160
        ERLARG = ERLARG-ERLAST
        IF(LEVCUR+1.LE.LEVMAX) ERLARG = ERLARG+ERRO12
        IF(EXTRAP) GO TO 120
C
C           TEST WHETHER THE INTERVAL TO BE BISECTED NEXT IS THE
C           SMALLEST INTERVAL.
C
        IF(LEVEL(MAXERR)+1.LE.LEVMAX) GO TO 160
        EXTRAP = .TRUE.
        NRMAX = 2
  120   IF(IERRO.EQ.3.OR.ERLARG.LE.ERTEST) GO TO 140
C
C           THE SMALLEST INTERVAL HAS THE LARGEST ERROR.
C           BEFORE BISECTING DECREASE THE SUM OF THE ERRORS OVER THE
C           LARGER INTERVALS (ERLARG) AND PERFORM EXTRAPOLATION.
C
        ID = NRMAX
        JUPBND = LAST
        IF(LAST.GT.(2+LIMIT/2)) JUPBND = LIMIT+3-LAST
        DO 130 K = ID,JUPBND
          MAXERR = IORD(NRMAX)
          ERRMAX = ELIST(MAXERR)
C***JUMP OUT OF DO-LOOP
          IF(LEVEL(MAXERR)+1.LE.LEVMAX) GO TO 160
          NRMAX = NRMAX+1
  130 CONTINUE

C
C           PERFORM EXTRAPOLATION.
C
  140   NUMRL2 = NUMRL2+1
        RLIST2(NUMRL2) = AREA
        IF(NUMRL2.LE.2) GO TO 155
        CALL QEXT(NUMRL2,RLIST2,RESEPS,ABSEPS,RES3LA,NRES)
        KTMIN = KTMIN+1
        IF(KTMIN.GT.5.AND.ABSERR.LT.1.0D-03*ERRSUM) IER = 5
        IF(ABSEPS.GE.ABSERR) GO TO 150
        KTMIN = 0
        ABSERR = ABSEPS
        RESULT = RESEPS
        CORREC = ERLARG
        ERTEST = DMAX1(EPSABS,EPSREL*DABS(RESEPS))
C***JUMP OUT OF DO-LOOP
        IF(ABSERR.LE.ERTEST) GO TO 170
C
C           PREPARE BISECTION OF THE SMALLEST INTERVAL.
C
  150   IF(NUMRL2.EQ.1) NOEXT = .TRUE.
        IF(IER.EQ.5) GO TO 170
  155   MAXERR = IORD(1)
        ERRMAX = ELIST(MAXERR)
        NRMAX = 1
        EXTRAP = .FALSE.
        LEVMAX = LEVMAX+1
        ERLARG = ERRSUM
  160 CONTINUE
C
C           SET THE FINAL RESULT.
C           ---------------------
C
  170 IF(ABSERR.EQ.OFLOW) GO TO 190
      IF((IER+IERRO).EQ.0) GO TO 180
      IF(IERRO.EQ.3) ABSERR = ABSERR+CORREC
      IF(IER.EQ.0) IER = 3
      IF(RESULT.NE.0.0D+00.AND.AREA.NE.0.0D+00) GO TO 175
      IF(ABSERR.GT.ERRSUM) GO TO 190
      IF(AREA.EQ.0.0D+00) GO TO 210
      GO TO 180
  175 IF(ABSERR/DABS(RESULT).GT.ERRSUM/DABS(AREA)) GO TO 190
C
C           TEST ON DIVERGENCE
C
  180 IF(KSGN.EQ.(-1).AND.DMAX1(DABS(RESULT),DABS(AREA)).LE.
     * DEFABS*1.0D-02) GO TO 210
      IF(1.0D-02.GT.(RESULT/AREA).OR.(RESULT/AREA).GT.1.0D+02.
     * OR.ERRSUM.GT.DABS(AREA)) IER = 6
      GO TO 210
C
C            COMPUTE GLOBAL INTEGRAL SUM.
C
  190 RESULT = 0.0D+00
      DO 200 K = 1,LAST
        RESULT = RESULT+RLIST(K)
  200 CONTINUE
      ABSERR = ERRSUM
  210 IF(IER.GT.2) IER = IER-1
      RESULT = RESULT*SIGN
  999 RETURN
      END
