$DEBUG
      SUBROUTINE QAGS(F,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
C
C.....................................................................
C
C 1.	QAGS
C	COMPUTATION OF A DEFINITE INTEGRAL
C	   STANDARD FORTRAN SUBROUTINE
C
C 2.	PURPOSE
C          THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C	DEFINITE INTEGRAL   I = INTEGRAL OF F OVER (A,B),
C	HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C	ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.	CALLING SEQUENCE
C	   CALL QAGS(F,A,B,EPSABS,EPSREL,RESULT,ABSER,NEVAL,IER)
C
C	PARAMETERS
C        ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF  EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C	ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR RESULT AND ERRORS ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                        = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE VALUE OF
C                            LIMIT IN QAGS (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENT INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT,
C                            IT IS ADVISED TO ANALYSE THE
C                            INTEGRAND IN ORDER TO DETERMINE THE
C                            INTEGRATION DIFFICULTIES. IF THE POSITION
C                            OF A LOCAL DIFFICULTY CAN BE DETERMINED
C                            (E.G. SINGULARITY, DISCONTINUITY WITHIN
C                            THE INTERVAL) ONE WILL PROBABLY GAIN FROM
C                            SPLITTING UP THE INTERVAL AT THIS POINT
C                            AND CALLING THE INTEGRATOR ON THE
C                            SUBRANGES. IF POSSIBLE, AN APPROPRIATE
C                            SPECIAL-PURPOSE INTEGRATOR SHOULD BE USED
C                            WHICH IS DESIGNED FOR HANDLING THE TYPE OF
C                            DIFFICULTY INVOLVED.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                            THE ERROR MAY BE UNDER-ESTIMATED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 4 THE ALGORITHM DOES NOT CONVERGE. ROUNDOFF
C                            ERROR IS DETECTED IN THE EXTRAPOLATION
C                            TABLE. IT IS PRESUMED THAT THE REQUESTED
C                            TOLERANCE CANNOT BE ACHIEVED, AND THAT THE
C                            RETURNED RESULT IS THE BEST WHICH CAN BE
C                            OBTAINED.
C                        = 5 THE INTEGRAL IS PROBABLY DIVERGENT, OR
C                            SLOWLY CONVERGENT. IT MUST BE NOTED THAT
C                            DIVERGENCE CAN OCCUR WITH ANY OTHER VALUE
C                            OF IER.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR AND NEVAL ARE SET TO ZERO.
C
C 4.	SUBROUTINES OR FUNCTIONS NEEDED
C             - QK21
C             - QSORT
C             - F (USER-PROVIDED FUNCTION)
C             - QMACO
C             - FORTRAN DABS, DMAX1, DMIN1
C
C....................................................................
C
      REAL*8
     *A,ABSEPS,ABSERR,ALIST,AREA,AREA1,AREA12,AREA2,A1,A2,B,BLIST,
     *  B1,B2,CORREC,DEFABS,DEFAB1,DEFAB2,DRES,ELIST,EPMACH,EPSABS,
     *  EPSREL,ERLARG,ERLAST,ERBND,ERRMAX,ERROR1,ERROR2,ERRO12,ERRSUM,
     *  ERTEST,F,OFLOW,RESABS,RESEPS,RESULT,RES3LA,RLIST,RLIST2,SMALL,
     *  UFLOW
      INTEGER ID,IER,IERRO,IORD,IROFF1,IROFF2,JUPBND,K,KSGN,
     *  KTMIN,LAST,LIMIT,MAXERR,NEVAL,NRES,NRMAX,NUMRL2
      LOGICAL EXTRAP,NOEXT
C
      DIMENSION ALIST(500),BLIST(500),ELIST(500),IORD(500),RES3LA(3),
     *  RLIST(500),RLIST2(52)
C
C	THE DIMENSION OF RLIST2 IS DETERMINED BY THE VALUE OF
C	LIMEXP IN SUBROUTINE QEXT (RLIST2 SHOULD BE OF DIMENSION
C       (LIMEXP+2) AT LEAST)
C
      EXTERNAL F
C
C	LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN THE
C	SUBDIVISION PROCESS. TAKE CARE THAT LIMIT.GE.1.
C
	DATA LIMIT/500/
C
C            LIST OF MAJOR VARIABLES
C            -----------------------
C
C           ALIST     - LIST OF LEFT END POINTS OF ALL SUBINTERVALS
C                       CONSIDERED UP TO NOW
C           BLIST     - LIST OF RIGHT END POINTS OF ALL SUBINTERVALS
C                       CONSIDERED UP TO NOW
C           RLIST(I)  - APPROXIMATION TO THE INTEGRAL OVER
C                       (ALIST(I),BLIST(I))
C           RLIST2    - ARRAY OF DIMENSION AT LEAST LIMEXP+2 CONTAINING
C                       THE PART OF THE EPSILON TABLE WHICH IS STILL
C                       NEEDED FOR FURTHER COMPUTATIONS
C           ELIST(I)  - ERROR ESTIMATE APPLYING TO RLIST(I)
C           MAXERR    - POINTER TO THE INTERVAL WITH LARGEST ERROR
C                       ESTIMATE
C           ERRMAX    - ELIST(MAXERR)
C           ERLAST    - ERROR ON THE INTERVAL CURRENTLY SUBDIVIDED
C                       (BEFORE THAT SUBDIVISION HAS TAKEN PLACE)
C           AREA      - SUM OF THE INTEGRALS OVER THE SUBINTERVALS
C           ERRSUM    - SUM OF THE ERRORS OVER THE SUBINTERVAL
C           ERRBND    - REQUESTED ACCURACY MAX(EPSABS,EPSREL*
C                       ABS(RESULT))
C           *****1    - VARIABLE FOR THE LEFT INTERVAL
C           *****2    - VARIABLE FOR THE RIGHT INTERVAL
C           LAST      - INDEX FOR SUBDIVISION
C           NRES      - NUMBER OF CALLS TO THE EXTRAPOLATION ROUTINE
C           NUMRL2    - NUMBER OF ELEMENTS CURRENTLY IN RLIST2. IF AN
C                       APPROPRIATE APPROXIMATION TO THE COMPOUNDED
C                       INTEGRAL HAS BEEN OBTAINED IT IS PUT IN
C                       RLIST2(NUMRL2) AFTER NUMRL2 HAS BEEN INCREASED
C                       BY ONE.
C           SMALL     - LENGTH OF THE SMALLEST INTERVAL CONSIDERED
C                       UP TO NOW, MULTIPLIED BY 1.5
C           ERLARG    - SUM OF THE ERRORS OVER THE INTERVALS LARGER
C                       THAN THE SMALLEST INTERVAL CONSIDERED UP TO
C                       NOW.
C           EXTRAP    - LOGICAL VARIABLE DENOTING THAT THE ROUTINE IS
C                       ATTEMPTING TO PERFORM EXTRAPOLATION I.E. BEFORE
C                       SUBDIVIDING THE SMALLEST INTERVAL WE TRY TO
C                       DECREASE THE VALUE OF ERLARG.
C           NOEXT     - LOGICAL VARIABLE DENOTING THAT EXTRAPOLATION
C                       IS NO LONGER ALLOWED (TRUE VALUE).
C
C
C   	    MACHINE DEPENDENT CONSTANTS
C           ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST POSITIVE MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
	CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C	TEST ON VALIDITY OF PARAMETERS
C       ------------------------------
C
	IER = 0
        NEVAL = 0
        LAST = 0
        RESULT = 0.0D+00
        ABSERR = 0.0D+00
        ALIST(1) = A
        BLIST(1) = B
        RLIST(1) = 0.0D+00
        ELIST(1) = 0.0D+00
        IF(EPSABS.LT.0.D0.AND.EPSREL.LT.0.D0) IERR = 6
        IF(IER.EQ.6) GO TO 999
C
C	FIRST APPROXIMATION TO THE INTEGRAL
C       -----------------------------------
C
	IERRO = 0
        CALL QK21(F,A,B,RESULT,ABSERR,DEFABS,RESABS)
C
C	TEST ON ACCURACY
C
       DRES = DABS(RESULT)
       ERRBND = DMAX1(EPSABS,EPSREL*DRES)
       LAST=1
       RLIST(1) = RESULT
       ELIST(1) = ABSERR
       IORD(1) = 1
       IF(ABSERR.LE.1.D+02*EPMACH*DEFABS.AND.ABSERR.GT.ERRBND) IER = 2
       IF(LIMIT.EQ.1) IER = 1
       IF(IR.NE.0.OR.(ABSERR.LE.ERRBND.AND.ABSERR.NE.RESABS)
     *   .OR.ABSERR.EQ.0.D+00) GO TO 140
C
C	INITIALIZATION
C       --------------
C
C
	RLIST2(1) = RESULT
	ERRMAX = ABSERR
        MAXERR = 1
        AREA = RESULT
        ERRSUM = ABSERR
        ABSERR = OFLOW
        NRMAX = 1
        NRES = 0
        NUMRL2 = 2
        KTMIN = 0
        EXTRAP = .FALSE.
        NOEXT = .FALSE.
        IROFF1 = 0
        IROFF2 = 0
        IPROF3 = 0
        KSGN = -1
        IF(DRES.GE.(1.0D00-5.D+1*EPMACH)*DEFABS) KSGN = 1
C
C	MAIN DO-LOOP
C       ------------
C
	DO 90 LAST = 2,LIMIT
C
C	BISECT THE SUBINTERVAL WITH THE NRMAX-TH LARGEST ERROR
C       ESTIMATE
C
	A1 = ALIST(MAXERR)
        B1 = 5.D-01*(ALIST(MAXERR)+BLIST(MAXERR))
        A2 = B1
        B2 = BLIST(MAXERR)
        ERLAST = ERRMAX
        CALL QK21(F,A1,B1,AREA1,ERROR1,RESABS,DEFAB1)
        CALL QK21(F,A2,B2,AREA2,ERROR2,RESABS,DEFAB2)
C
C	  IMPROVE PREVIOUS APPROXIMATIONS TO INTEGRAL AND ERROR AND
C	  TEST FOR ACCURACY
C
        AREA12 = AREA1+AREA2
        ERRO12 = ERROR1+ERROR2
        ERRSUM = ERRSUM+ERRO12-ERRMAX
        AREA = AREA+AREA12-RLIST(MAXERR)
        IF(DEFAB1.EQ.ERROR1.OR.DEFAB2.EQ.ERROR2) GO TO 15
        IF(ABS(RLIST(MAXERR)-AREA12).GT.1.0D-05*DABS(AREA12)
     *    .OR.ERRO12.LT.9.9D-01*ERRMAX) GO TO 10
     	IF(EXTRAP) IROFF2 = IROFF2+1
        IF(.NOT.EXTRAP) IROFF1 = IROFF1+1
   10   IF(LAST.GT.10.AND.ERRO12.GT.ERRMAX) IROFF3 = IROFF3+1
   15	RLIST(MAXERR) = AREA1
    	RLIST(LAST) = AREA2
        ERRBND = DMAX1(EPSABS,EPSREL*DABS(AREA))
C
C	TEST FOR ROUNDOFF ERROR AND EVENTUALLY SET ERROR FLAG
C
	IF(IROFF1+IROFF2.GE.10.OR.IROFF3.GE.20) IER = 2
        IF(IROFF2.GE.5) IERRO = 3
C
C    SET ERROR FLAG IN THE CASE THAT THE NUMBER OF SUBINTERVALS
C    EQUALS LIMIT
C
	IF(LAST.EQ.LIMIT) IER = 1
C
C     SET ERROR FLAG IN THE CASE OF BAD INTEGRAND BEHAVIOUR
C     AT A POINT OF THE INTEGRATION RANGE.
C
      IF(DMAX1(DABS(A1),DABS(B2)).LE.(1.D0+C*1.0D+03*
     * EPMACH)*(DABS(A2)+1.0D+04*UFLOW)) IER = 4
C
C    APPEND THE NEWLY-CREATED INTERVALS TO THE LIST
C
      IF(ERROR2.GT.ERROR1) GO TO 20
    	ALIST(LAST) = A2
        BLIST(MAXERR) = B1
        BLIST(LAST) = B2
        ELIST(MAXERR) = ERROR1
        ELIST(LAST) = ERROR2
        GO TO 30
  20	ALIST(MAXERR) = A2
        ALIST(LAST) = A1
        BLIST(LAST) = B1
        RLIST(MAXERR) = AREA2
        RLIST(LAST) = AREA1
        ELIST(MAXERR) = ERROR2
        ELIST(LAST) = ERROR1
C
C	CALL SUBROUTINE QSQRT TO MAINTAIN THE DESCENDING ORDERING
C       IN THE LIST OF ERROR ESTIMATES AND SELECT THE SUBINTERVAL
C	WITH NRMAX-TH LARGEST ERROR ESTIMATE (TO BE BISECTED NEXT).
C
   30	CALL QSORT(LIMIT,LAST,MAXERR,ERRMAX,ELIST,IORD,NRMAX)
C***JUMP OUT OF DO-LOOP
	IF(ERRSUM.LE.ERRBND) GO TO 115
C***JUMP OUT OF DO-LOOP
	IF(IER.NE.0.) GO TO 100
        IF(LAST.EQ.2) GO TO 80
        IF(NOEXT) GO TO 90
        ERLARG = ERLARG-ERLAST
        IF(DABS(B1-A1).GT.SMALL) ERLARG = ERLARG+ERRO12
        IF(EXTRAP) GO TO 40
C
C	TEST WHETHER THE INTERVAL TO BE BISECTED NEXT IS THE
C	SMALLEST INTERVAL
C
	IF(DABS(BLIST(MAXERR)-ALIST(MAXERR)).GT.SMALL) GO TO 90
        EXTRAP = .TRUE.
        NRMAX = 2
   40	IF(IERRO.EQ.3.OR.ERLARG.LE.ERTEST) GO TO 60
C
C	THE SMALLEST INTERVAL HAS THE LARGEST ERROR
C	BEFORE BISECTING DECREASE THE SUM OF THE ERRORS OVER THE
C	LARGER INTERVALS (ERLARG) AND PERFORM EXTRAPOLATION
C
	ID = NRMAX
        JUPBND = LAST
        IF(LAST.GT.(2+LIMIT/2)) JUPBND = LIMIT + 3 - LAST
        DO 50 K = ID,JUPBND
           MAXERR = IORD(NRMAX)
           ERRMAX = ELIST(MAXERR)
C***JUMP OUT OF DO-LOOP
           IF(DABS(BLIST(MAXERR)-ALIST(MAXERR)).GT.SMALL) GO TO 90
           NRMAX = NRMAX+1
   50	CONTINUE
C
C	PERFORM EXTRAPOLATION
C
   60	NUMRL2 = NUMRL2+1
   	RLIST2(NUMRL2) = AREA
        CALL QEXT(NUMRL2,RLIST2,RESEPS,ABSEPS,RES3LA,NRES)
        KTMIN = KTMIN + 1
        IF(KTMIN.GT.5.AND.ABSERR.LT.1.0D-03*ERRSUM) IER = 5
        IF(ABSEPS.GE.ABSERR) GO TO 70
        KTMIN = 0
        ABSERR = ABSEPS
        RESULT = RESEPS
        CORREC = ERLARG
        ERTEST = DMAX1(EPSABS,EPSREL*DABS(RESEPS))
C***JUMP OUT OF DO-LOOP
	IF(ABSERR.LE.ERTEST) GO TO 100
C
C	PREPARE BISECTION OF THE SMALLEST INTERVAL
C
   70	IF(NUMRL2.EQ.1) NOEXT = .TRUE.
        IF(IER.EQ.5) GO TO 100
        MAXERR = IORD(1)
        ERRMAX = ELIST(MAXERR)
        NRMAX = 1
        EXTRAP = .FALSE.
        SMALL = SMALL*5.0D-01
        ERLARG = ERRSUM
        GO TO 90
   80 	SMALL = DABS(B-A)*3.75D-01
   	ERLARG = ERRSUM
        ERTEST = ERRBND
        RLIST2(2) = AREA
   90	CONTINUE
C
C	SET FINAL RESULT AND ERROR ESTIMATE
C       -----------------------------------
  100	IF(ABSERR.EQ.OFLOW) GO TO 115
        IF(IER+IERRO.EQ.0) GO TO 110
        IF(IERRO.EQ.3) ABSERR = ABSERR + CORREC
        IF(IER.EQ.0) IER = 3
        IF(RESULT.NE.0.D0.AND.AREA.NE.0.D0) GO TO 105
        IF(ABSERR.GT.ERRSUM) GO TO 115
        IF(AREA.EQ.0.D0) GO TO 130
        GO TO 110
  105	IF(ABSERR/DABS(RESULT).GT.ERRSUM/DABS(AREA)) GO TO 115
C
C	TEST ON DIVERGENCE
C
  110	IF(KSGN.EQ.(-1).AND.DMAX1(DABS(RESULT),DABS(AREA)).LE.
     * DEFABS*1.D-02) GO TO 130
     	IF(1.D-02.GT.(RESULT/AREA).OR.(RESULT/AREA).GT.1.D+2
     * .OR.ERRSUM.GT.DABS(AREA)) IER = 6
     	GO TO 130
C
C	COMPUTE GLOBAL INTEGRAL SUM.
C
  115	RESULT = 0.D0
  	DO 120 K=1,LAST
           RESULT = RESULT+RLIST(K)
  120	CONTINUE
  	ABSERR = ERRSUM
  130	IF(IER.GT.2) IER = IER-1
  140	NEVAL = 42*LAST-21
  999	RETURN
   	END
