$debug
      SUBROUTINE QAWC(F,A,B,C,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
C
C......................................................................
C
C 1.     QAWC
C        COMPUTATION OF A CAUCHY PRINCIPAL VALUE
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A
C           CAUCHY PRINCIPAL VALUE I = INTEGRAL OF F*W OVER (A,B)
C           (W(X) = 1/(X-C), C.NE.A, C.NE.B), HOPEFULLY SATISFYING
C           FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.     CALLING SEQUENCE
C           CALL QAWC(F,A,B,C,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           C      - PARAMETAR IN THE WEIGHT FUNCTION, C.NE.A, C.NE.B
C                    IF C = A OR C = B, THE ROUTINE WILL END WITH
C                    IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OR THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR INTEGRAL AND ERROR ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALOW MORE SUB-
C                            DIVISIONS BY INCREASING THE DATA VALUE OF
C                            LIMIT IN QAWC (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENTS INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT IT
C                            IS ADVISED TO ANALYZE THE INTEGRAND IN
C                            ORDER TO DETERMINE THE INTEGRATION
C                            DIFFICULTIES. IF THE POSITION OF A LOCAL
C                            DIFFICULTY CAN BE DETERMINED (E.G.
C                            SINGULARITY, DISCONTINUITY WITHIN THE
C                            INTERVAL) ONE WILL PROBABLY GAIN FROM
C                            SPLITTING UP THE INTERVAL AT THIS POINT
C                            AND CALLING APPROPRIATE INTEGRATORS ON
C                            SUBRANGES.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS DETEC-
C                            TED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR
C                            OCCURSAT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            C = A OR C = B OR
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR, NEVAL ARE SET TO ZERO.
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C              - QAWCE
C              - QC25C
C              - QSORT
C              - QK15W
C              - QCHEB
C              - F (USER-PROVIDED FUNCTION)
C              - QWGTC
C              - QMACO
C              - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,ABSERR,ALIST,B,BLIST,ELIST,C,EPSABS,EPSREL,F,RESULT,RLIST
      INTEGER IER,IORD,LAST,LIMIT,NEVAL
C
      DIMENSION ALIST(500),BLIST(500),ELIST(500),IORD(500),RLIST(500)
C
      EXTERNAL F
C
C         LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN THE
C         SUBDIVISION PROCESS OF QAWCE. TAKE CARE THAT LIMIT.GE.1.
C
      DATA LIMIT/500/
C
C***FIRST EXECUTABLE STATEMENT
      CALL QAWCE(F,A,B,C,EPSABS,EPSREL,LIMIT,RESULT,ABSERR,NEVAL,IER,
     *  ALIST,BLIST,RLIST,ELIST,IORD,LAST)
      RETURN
      END
