$debug
      SUBROUTINE QAWCE(F,A,B,C,EPSABS,EPSREL,LIMIT,RESULT,ABSERR,NEVAL,
     *  IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
C......................................................................
C
C 1.     QAWCE
C        COMPUTATION OF A CAUCHY PRINCIPAL VALUE
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A
C           CAUCHY PRINCIPAL VALUE I = INTEGRAL OF F*W OVER (A,B)
C           (W(X) = 1/(X-C), (C.NE.A, C.NE.B), HOPEFULLY SATISFYING
C           FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.     CALLING SEQUENCE
C           CAL QAWCE(F,A,B,C,EPSABS,EPSREL,LIMIT,RESULT,ABSERR,NEVAL,
C                     IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C           C      - REAL
C                    PARAMETER IN THE WEIGHT FUNCTION, C.NE.A OR C = B
C                    IF C= A OR C = B, THE ROUTINE WIL END WITH
C                    IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF EPSABS.LT.0 AND EPSREL.LT.O,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C           LIMIT - INTEGER
C                   GIVES EN UPPER BOUND ON THE NUMBER OF SUBINTERVALS
C                   IN THE PARTITION OF (A,B), LIMIT.GE.1
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR INTEGRAL AND ERROR ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALOW MORE SUB-
C                            DIVISIONS BY INCREASING THE VALUE OF
C                            LIMIT. HOWEVER, IF THIS YIELDS NO
C                            IMPROVEMENT IT IS ADVISED TO ANALYZE THE
C                            INTEGRAND, IN ORDER TO DETERMINE THE
C                            INTEGRATION DIFFICULTIES. IF THE POSITION
C                            OF A LOCAL DIFFICULTY CAN BE DETERMINED
C                            (E.G. SINGULARITY, DISCONTINUITY WITHIN
C                            THE INTERVAL) ONE WILL PROBABLY GAIN
C                            FROM SPLITTING AT THIS
C                            POINT AND CALLING APPROPRIATE INTEGRATORS
C                            ON THE SUBRANGES.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS DETEC-
C                            TED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME INTERIOR POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            C = A OR C = B OR
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            OR LIMIT.LT.1.
C                            RESULT, ABSERR, NEVAL, RLIST(1), ELIST(1),
C                            IORD(1) AND LAST ARE SET TO ZERO.
C                            ALIST(1) AND BLIST(1) ARE SET TO A AND B
C                            RESPECTIVELY.
C
C           ALIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT,THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE LEFT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B)
C
C           BLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE RIGHT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B)
C
C           RLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                    LAST ELEMENTS OF WHICH ARE THE INTEGRAL
C                    APPROXIMATIONS ON THE SUBINTERVALS
C
C           ELIST  - REAL
C                    VECTOR OF DIMENSION LIMIT, THE FIRST LAST
C                    ELEMENTS OF WHICH ARE THE MODULI OF THE ABSOLUTE
C                    ERROR ESTIMATES ON THE SUBINTERVALS
C
C           IORD   - INTEGER
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST  K
C                    ELEMENTS OF WHICH ARE POINTERS TO THE ERROR
C                    ESTIMATES OVER THE SUBINTERVALS, SO THAT
C                    ELIST(IORD(1)), ..., ELIST(IORD(K)) WITH
C                    K = LAST IF LAST.LE.(LIMIT/2+2), AND
C                    K = LIMIT+1-LAST OTHERWISE, FORM A DECREASING
C                    SEQUENCE.
C
C           LAST   - INTEGER
C                    NUMBER OF SUBINTERVALS ACTUALLY PRODUCED IN
C                    THE SUBDIVISION PROCESS
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C             - QC25C
C             - QSORT
C             - QK15W
C             - QCHEB
C             - QWGTC
C             - QMACO
C             - F (USER-PROVIDED FUNCTION)
C             - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,AA,ABSERR,ALIST,AREA,AREA1,AREA12,AREA2,A1,A2,B,BB,BLIST,
     *  B1,B2,C,ELIST,EPMACH,EPSABS,EPSREL,ERRBND,ERRMAX,ERROR1,ERROR2,
     *  ERRO12,ERRSUM,F,OFLOW,RESULT,RLIST,UFLOW
      INTEGER IER,IORD,IROFF1,IROFF2,K,KRULE,LAST,LIMIT,MAXERR,NEV,
     *  NEVAL,NRMAX
C
      DIMENSION ALIST(LIMIT),BLIST(LIMIT),RLIST(LIMIT),ELIST(LIMIT),
     *  IORD(LIMIT)
C
      EXTERNAL F
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           ALIST    - LIST OF LEFT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           BLIST    - LIST OF RIGHT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           RLIST(I) - APPROXIMATION TO THE INTEGRAL OVER
C                      (ALIST(I),BLIST(I))
C           ELIST(I) - ERROR ESTIMATE APPLYING TO RLIST(I)
C           MAXERR   - POINTER TO THE INTERVAL WITH LARGEST ERROR
C                      ESTIMATE
C           ERRMAX   - ELIST(MAXERR)
C           AREA     - SUM OF THE INTEGRALS OVER THE SUBINTERVALS
C           ERRSUM   - SUM OF THE ERRORS OVER THE SUBINTERVALS
C           ERRBND   - REQUESTED ACCURACY MAX(EPSABS,EPSREL*
C                      ABS(RESULT))
C           *****1   - VARIABLE FOR THE LEFT SUBINTERVAL
C           *****2   - VARIABLE FOR THE RIGHT SUBINTERVAL
C           LAST     - INDEX FOR SUBDIVISION
C
C
C           MACHINE DEPENDENT COSTANTS
C           --------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C           TEST OF VALIDITY OF PARAMETERS
C           ------------------------------
C
      IER = 6
      NEVAL = 0
      LAST = 0
      ALIST(1) = A
      BLIST(1) = B
      RLIST(1) = 0.0D+00
      ELIST(1) = 0.0D+00
      IORD(1) = 0
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      IF(C.EQ.A.OR.C.EQ.B.OR.(EPSABS.LT.0.0D+00.AND.EPSREL.LT.0.0D+00))
     *  GO TO 999
C
C           FIRST APPROXIMATION TO THE INTEGRAL
C           -----------------------------------
C
      AA = A
      BB = B
      IF(A.LE.B) GO TO 10
      AA = B
      BB = A
   10 IER = 0
      KRULE = 1
      CALL QC25C(F,AA,BB,C,RESULT,ABSERR,KRULE,NEVAL)
      LAST = 1
      RLIST(1) = RESULT
      ELIST(1) = ABSERR
      IORD(1) = 1
      ALIST(1) = A
      BLIST(1) = B
C
C           TEST ON ACCURACY
C
      ERRBND = DMAX1(EPSABS,EPSREL*DABS(RESULT))
      IF(LIMIT.EQ.1) IER = 1
      IF(ABSERR.LT.DMIN1(1.0D-02*DABS(RESULT),ERRBND).OR.IER.EQ.1)
     *  GO TO 70
C
C           INITIALIZATION
C           --------------
C
      ALIST(1) = AA
      BLIST(1) = BB
      RLIST(1) = RESULT
      ERRMAX = ABSERR
      MAXERR = 1
      AREA = RESULT
      ERRSUM = ABSERR
      NRMAX = 1
      IROFF1 = 0
      IROFF2 = 0
C
C           MAIN DO-LOP
C           -----------
C
      DO 40  LAST = 2,LIMIT
C
C           BISECT THE SUBINTERVAL WITH NRMAX-TH LARGEST ERROR ESTIMATE.
C
        A1 = ALIST(MAXERR)
        B1 = 5.0D-01*(ALIST(MAXERR)+BLIST(MAXERR))
        B2 = BLIST(MAXERR)
        IF(C.LE.B1.AND.C.GT.A1) B1 = 5.0D-01*(C+B2)
        IF(C.GT.B1.AND.C.LT.B2) B1 = 5.0D-01*(A1+C)
        A2 = B1
        KRULE = 2
        CALL QC25C(F,A1,B1,C,AREA1,ERROR1,KRULE,NEV)
        NEVAL = NEVAL+NEV
        CALL QC25C(F,A2,B2,C,AREA2,ERROR2,KRULE,NEV)
        NEVAL = NEVAL+NEV
C
C           IMPROVE PREVIOUS APPROXIMATIONS TO INTEGRAL AND ERROR
C           AND TEST FOR ACCRACY.
C
        AREA12 = AREA1+AREA2
        ERRO12 = ERROR1+ERROR2
        ERRSUM = ERRSUM+ERRO12-ERRMAX
        AREA = AREA+AREA12-RLIST(MAXERR)
        IF(DABS(RLIST(MAXERR)-AREA12).LT.1.0D-05*DABS(AREA12)
     *    .AND.ERRO12.GE.9.9D-01*ERRMAX.AND.KRULE.EQ.0)
     *    IROFF1 = IROFF1+1
        IF(LAST.GT.10.AND.ERRO12.GT.ERRMAX.AND.KRULE.EQ.0)
     *    IROFF2 = IROFF2+1
        RLIST(MAXERR) = AREA1
        RLIST(LAST) = AREA2
        ERRBND = DMAX1(EPSABS,EPSREL*DABS(AREA))
        IF(ERRSUM.LE.ERRBND) GO TO 15
C
C           TEST FOR ROUNDOFF ERROR AND EVENTUALLY SET ERROR FLAG.
C
        IF(IROFF1.GE.6.AND.IROFF2.GT.20) IER = 2
C
C           SET ERROR FLAG IN THE CASE THAT NUMBER OF INTERVAL
C           BISECTIONS EXCEEDS LIMIT.
C
        IF(LAST.EQ.LIMIT) IER = 1
C
C           SET ERROR FLAG IN THE CASE OF BAD INTEGRAND BEHAVIOUR AT
C           A POINT OF THE INTEGRATION RANGE.
C
        IF(DMAX1(DABS(A1),DABS(B2)).LE.(1.0D+00+1.0D+03*EPMACH)
     *    *(DABS(A2)+1.0D+03*UFLOW)) IER = 3
C
C           APPEND THE NEWLY-CREATED INTERVALS TO THE LIST.
C
   15   IF(ERROR2.GT.ERROR1) GO TO 20
        ALIST(LAST) = A2
        BLIST(MAXERR) = B1
        BLIST(LAST) = B2
        ELIST(MAXERR) = ERROR1
        ELIST(LAST) = ERROR2
        GO TO 30
   20   ALIST(MAXERR) = A2
        ALIST(LAST) = A1
        BLIST(LAST) = B1
        RLIST(MAXERR) = AREA2
        RLIST(LAST) = AREA1
        ELIST(MAXERR) = ERROR2
        ELIST(LAST) = ERROR1
C
C           CALL SUBROUTINE QSORT TO MAINTAIN THE DESCENDING ORDERING
C           IN THE LIST OF ERROR ESTIMATES AND SELECT THE SUBINTERVAL
C           WITH NRMAX-TH LARGEST EROR ESTIMATE (TO BE BISECTED NEXT).
C
   30   CALL QSORT(LIMIT,LAST,MAXERR,ERRMAX,ELIST,IORD,NRMAX)
C***JUMP OUT OF DO-LOP
        IF(IER.NE.0.OR.ERRSUM.LE.ERRBND) GO TO 50
   40 CONTINUE
C
C           COMPUTE FINAL RESULT.
C           --------------------
C
   50 RESULT = 0.0D+00
      DO 60 K=1,LAST
        RESULT = RESULT+RLIST(K)
   60 CONTINUE

      ABSERR = ERRSUM
   70 IF(AA.EQ.B) RESULT = -RESULT
  999 RETURN
      END

