      SUBROUTINE QAWF(F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,IER)
C
C......................................................................
C
C 1.     QAWF
C        COMPUTATION OF FOURIER INTEGRALS
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C           FOURIER INTEGRAL
C                I = INTEGRAL OF F(X)*W(X) OVER (A,INFINITY)
C                    WHERE W(X) = COS(OMEGA*X)
C                       OR W(X) = SIN(OMEGA*X),
C           HOPEFULLY SATISFAYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.EPSABS.
C
C 3.     CALLING SEQUENCE
C           CALL QAWF(F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,IER)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           OMEGA  - REAL
C                    PARAMETER IN THE INTEGRAND WEIGHT FUNCTION
C
C           INTEGR - INTEGER
C                    INDICATES WHICH OF THE WEIGHT FUNCTIONS IS USED
C                    INTEGR = 1      W(X) = COS(OMEGA*X)
C                    INTEGR = 2      W(X) = SIN(OMEGA*X)
C                    IF INTEGR.NE.1.AND.INTEGR.NE.2, THE ROUTINE
C                    WILL END WITH IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED, EPSABS.GT.0.
C                    IF EPSABS.LE.0, THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICHSHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE.
C                            THE ESTIMATES FOR INTEGRAL AND ERROR ARE
C                            LESS RELIABLE. IR IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                   IF OMEGA.NE.0
C                    IER = 6 THE INPUT IS INVALID BECAUSE
C                            (INTEGR.NE.1 AND INTEGR.NE2) OR
C                            EPSABS.LE.0
C                            RESULT, ABSERR, NEVAL, LST, ARE SET TO
C                            ZERO.
C                        = 7 ABNORMAL TERMINATION OF THE COMPUTATION
C                            OF ONE MORE SUBINTEGRALS
C                        = 8 MAXIMUM NUMBER OF CYCLES ALLOWED
C                            HAS BEEN ACHIEVED, I.E. OF SUBINTERVALS
C                            (A+(K-1)C,A+KC) WHERE
C                            C = (2*INT(ABS(OMEGA))+1)*PI/ABS(OMEGA),
C                            FOR K = 1,2, ...
C                        = 9 THE EXTRAPOLATION TABLE CONSTRUCTED FOR
C                            CONVERGENCE ACCELERATION OF THE SERIES
C                            FORMED BY THE INTEGRAL CONTRIBUTIONS
C                            OVER THE CYCLES, DOES NOT CONVERGE TO
C                            WITHIN THE REQUESTED ACCURACY.
C
C                   IF OMEGA = 0 AND INTEGR = 1,
C                   THE INTEGRAL IS CALCULATED BY MEANS OF QAGI
C                   AND IER HAS THE MEANING DESCRIBED IN THE
C                   COMMENTS OF QAGI
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C             - QAWFE
C             - QAGI
C             - QFOUR
C             - QEXT
C             - QK15I
C             - QSORT
C             - QC25O
C             - QK15W
C             - QCHEB
C             - QWGTO
C             - QMACO
C             - FORTRAN ABS, AMAX1, AMIN1
C             - F (USER-PROVIDED FUNCTION)
C
C......................................................................
C
      REAL*8 A,ABSERR,EPSABS,F,OMEGA,RESULT,ALIST,BLIST,ELIST,RLIST,
     *  RSLST,ERLST,CHEBMO
      INTEGER IER,INTEGR,LAST,LIMIT,LIMLST,IORD,IERLST,NNLOG,
     *  LST,MAXP1,NEVAL
C
      DIMENSION ALIST(500),BLIST(500),RLIST(500),ELIST(500),
     *  CHEBMO(21,25),ERLST(50),RSLST(50),IERLST(50),
     *  IORD(500),NNLOG(500)
C
      EXTERNAL F
C
C***FIRST EXECUTABLE STATEMENT
      IER = 6
      NEVAL = 0
      LAST = 0
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
C         DIMENSIONING PARAMETERS
C            LIMLST - INTEGER
C                     LIMLST GIVES AN UPPER BOUND ON
C                     THE NUMBER OF CYCLES, LIMLST.GE.3.
C                     IF LIMLST.LT.3, THE ROUTINE WILL END WITH IER = 6.
C
C            MAXP1 - INTEGER
C                    MAXP1 GIVES AN UPPER BOUND ON THE
C                    NUMBER OF CHEBYSHEV MOMENTS WHICH CAN BE
C                    STORED, I.E. FOR THE INTERVALS OF LENGTHS
C                    ABS(B-A)*2**(-L), L = 0,1, ..., MAXP1-2,
C                    MAXP1.GE.1.
C                    IF MAXP1.LT.1, THE ROUTINE WILL END WITH IER = 6.
      MAXP1 = 21
      LIMLST = 50
      LIMIT = 500
C
C         CHECK VALIDITY OF LIMLST AND MAXP1
C
      IF(LIMLST.LT.3.OR.MAXP1.LT.1) GO TO 10
C
C         PREPARE CALL FOR QAWFE
C
      CALL QAWFE(F,A,OMEGA,INTEGR,EPSABS,LIMLST,LIMIT,MAXP1,RESULT,
     *  ABSERR,NEVAL,IER,RSLST,ERLST,IERLST,LST,ALIST,
     *  BLIST,RLIST,ELIST,IORD,NNLOG,CHEBMO)
   10 RETURN
      END

