      SUBROUTINE QAWFE(F,A,OMEGA,INTEGR,EPSABS,LIMLST,LIMIT,MAXP1,
     *  RESULT,ABSERR,NEVAL,IER,RSLST,ERLST,IERLST,LST,ALIST,BLIST,
     *  RLIST,ELIST,IORD,NNLOG,CHEBMO)

C
C.....................................................................
C
C 1.     QAWFE
C        COMPUTATION OF FOURIER INTEGRALS
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C           FOURIER INTEGRAL
C                I = INTEGRAL OF F(X)*W(X) OVER (A,INFINITY)
C                    WHERE W(X) = COS(OMEGA*X)
C                       OR W(X) = SIN(OMEGA*X),
C           HOPEFULLY SATISFAYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.EPSABS.
C
C 3.     CALLING SEQUENCE
C           CALL QAWFE(F,A,OMEGA,INTEGR,EPSABS,LIMLST,LIMIT,MAXP1,
C                     RESULT,ABSERR,NEVAL,IER,RSLST,ERLST,IERLST,LST,
C                     ALIST,BLIST,RLIST,ELIST,IORD,NNLOG,CHEBMO)
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           OMEGA  - REAL
C                    PARAMETER IN THE INTEGRAND WEIGHT FUNCTION
C
C           INTEGR - INTEGER
C                    INDICATES WHICH OF THE WEIGHT FUNCTIONS IS USED
C                    INTEGR = 1      W(X) = COS(OMEGA*X)
C                    INTEGR = 2      W(X) = SIN(OMEGA*X)
C                    IF INTEGR.NE.1.AND.INTEGR.NE.2, THE ROUTINE
C                    WILL END WITH IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED, EPSABS.GT.0.
C                    IF EPSABS.LE.0, THE ROUTINE WILL END WITH IER = 6.
C
C           LIMLST - INTEGER
C                    LIMLST GIVES AN UPPER BOUND ON THE
C                    NUMBER OF CYCLES, LIMLST.GE.1.
C                    IF LIMLST.LT.3, THE ROUTINE WILL END WITH IER = 6.
C
C           LIMIT  - INTEGER
C                    GIVES AN UPPER BOUND ON THE NUMBER OF
C                    SUBINTERVALS ALLOWED IN THE PARTITION OF
C                    EACH CYCLE, LIMIT.GE.1.
C
C           MAXP1  - INTEGER
C                    GIVES AN UPPER BOUND ON THE NUMBER OF
C                    CHEBYSHEV MOMENTS WHICH CAN BE STORED, I.E.
C                    FOR THE INTERVALS OF LENGTHS ABS(B-A)*2**(-L),
C                    L=0,1, ..., MAXP1 - 2, MAXP1.GE.1
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICHSHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE.
C                            THE ESTIMATES FOR INTEGRAL AND ERROR ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                   IF OMEGA.NE.0
C                    IER = 6 THE INPUT IS INVALID BECAUSE
C                            (INTEGR.NE.1 AND INTEGR.NE.2) OR
C                            EPSABS.LE.0 OR LIMLST.LT.3.
C                            RESULT, ABSERR, NEVAL, LST, ARE SET TO
C                            TO ZERO.
C                        = 7 BAD INTEGRAND BEHAVIOUR OCCURS WITHIN
C                            ONE OR MORE OF THE CYCLES. LOCATION
C                            AND TYPE OF THE DIFFICULTY INVOLVED
C                            CAN BE DETERMINED FROM THE VECTOR IERLST.
C                            HERE LST IS THE NUMBER OF CYCLES ACTUALLY
C                            NEEDED (SEE BELOW).
C                        = 8 MAXIMUM NUMBER OF CYCLES ALLOWED
C                            HAS BEEN ACHIEVED., I.E. SUBINTERVALS
C                            (A+(K-1)C,A+KC) WHERE
C                            C = (2*INT(ABS(OMEGA))+1)*PI/ABS(OMEGA),
C                            FOR K = 1, 2, ..., LST.
C                            ONE CAN ALLOW MORE CYCLES BY INCREASING
C                            THE VALUE OF LIMLST (AND TAKING THE
C                            ACCORDING DIMENSION ADJUSTMENTS INTO
C                            ACCOUNT).
C                            EXAMINE THE ARRAY IERLST WHICH CONTAINS
C                            THE ERROR FLAGS ON THE CYCLES, IN ORDER
C                            TO EVENTUAL LOOK FOR LOCAL INTEGRATION
C                            DIFFICULTIES.
C                            IN THE POSITION OF A LOCAL DIFFICULTY CAN
C                            BE DETERMINED (E.G. SINGULARITY,
C                            DISCONTINUITY WITHIN THE INTERVAL)
C                            ONE WILL PROBABLY GAIN FROM SPLITTING
C                            UP THE INTERVAL AT THIS POINT AND
C                            CALLING APPROPRIATE INTEGRATORS ON
C                            THE SUBRANGES.
C                        = 9 THE EXTRAPOLATION TABLE CONSTRUCTED FOR
C                            CONVERGENCE ACCELERATION OF THE SERIES
C                            FORMED BY THE INTEGRAL CONTRIBUTIONS
C                            OVER THE CYCLES, DOES NOT CONVERGE TO
C                            WITHIN THE REQUESTED ACCURACY.
C                            AS IN THE CASE OF IER = 8, IT IS ADVISED
C                            TO EXAMINE THE ARRAY IERLST WHICH CONTAINS
C                            THE ERROR FLAGS ON THE CYCLES.
C                            IERLST(K) = 1 THE MAXIMUM NUMBER OF
C                                          SUBDIVISIONS (= LIMIT)
C                                          HAS BEEN ACHIEVED ON THE
C                                          K TH CYCLE.
C                                      = 2 OCCURENCE OF ROUNDOFF
C                                          ERROR IS DETECTED AND
C                                          PREVENTS THE TOLERANCE
C                                          IMPOSED ON THE K TH CYCLE
C                                          FROM BEING ACHIEVED.
C                                      = 3 EXTREMELY BAD INTEGRAND
C                                          BEHAVIOUR OCCURS AT SOME
C                                          POINTS OF THE K TH CYCLE.
C                                      = 4 THE INTEGRATION PROCEDURE
C                                          OVER THE K TH CYCLE DOES
C                                          NOT CONVERGE (TO WITHIN THE
C                                          REQUIRED ACCURACY) DUE TO
C                                          ROUNDOFF IN THE
C                                          EXTRAPOLATION PROCEDURE
C                                          INVOKED ON THIS CYCLE. IT
C                                          BEST WHICH CAN BE OBTAINED.
C                                      = 5 THE INTEGRAL OVER THE K TH
C                                          CYCLE IS PROBABLY DIVERGENT
C                                          OR SLOWLY CONVERGENT. IT
C                                          MUST BE NOTED THAT
C                                          DIVERGENCE CAN OCCUR WITH
C                                          ANY OTHER VALUE OF
C                                          IERLST(K).
C
C                   IF OMEGA = 0 AND INTEGR = 1,
C                   THE INTEGRAL IS CALCULATED BY MEANS OF QAGI
C                   AND IER HAS THE MEANING DESCRIBED IN THE
C                   COMMENTS OF QAGI
C
C           RSLST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMLST
C                    RSLST(K) CONTAINS THE INTEGRAL CONTRIBUTION
C                    OVER THE INTERVAL (A+(K-1)C,A+KC) WHERE
C                    C = (2*INT(ABS(OMEGA))+1)*PI/ABS(OMEGA),
C                    FOR K = 1, 2, ..., LST.
C                    NOTE THAT, IF OMEGA = 0, RSLST(1) CONTAINS
C                    THE VALUE OF THE INTEGRAL OVER (A,INFINITY).
C
C           ERLST - REAL
C                   VECTOR OF DIMENSION AT LEAST LIMLST
C                   ERLST(K) CONTAINS THE ERROR ESTIMATE
C                   CORRESPONDING WITH RSLST(K).
C
C           IERLST - INTEGER
C                    VECTOR OF DIMENSION AT LEAST LIMLST
C                    IERLST(K) CONTAINS THE ERROR FLAG CORRESPONDING
C                    WITH RSLST(K). FOR THE MEANING OF THE LOCAL ERROR
C                    FLAGS SEE DESCRIPTION OF OUTPUT PARAMETER IER.
C
C           LST    - INTEGER
C                    NUMBER OF SUBINTEGRALS NEDED FOR THE INTEGRATION
C                    IF OMEGA = 0 THEN LST IS SET TO 1.
C
C           ALIST, BLIST, RLIST, ELIST, - REAL
C                    WORKVECTORS OF DIMENSION AT LEAST LIMIT.
C
C           IORD, NNLOG, - INTEGER
C                    WORKVECTORS OF DIMENSION AT LEAST LIMIT, PROVIDING
C                    SPACE FOR THE QUANTITIES NEEDED IN THE
C                    SUBDIVISION PROCESS OF EACH CYCLE
C
C           CHEBMO - REAL
C                    WORKARRAY OF DIMENSION AT LEAST (MAXP1,25),
C                    PROVIDING SPACE FOR THE CHEBYSHEV MOMENTS
C                    NEEDED WITHIN THE CYCLES
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C             - QAGI
C             - QFOUR
C             - QEXT
C             - QK15I
C             - QSORT
C             - QC25O
C             - QK15W
C             - QCHEB
C             - QWGTO
C             - QMACO
C             - FORTRAN ABS, AMAX1, AMIN1
C             - F (USER-PROVIDED FUNCTION)
C
C......................................................................
C
      REAL*8 A,ABSEPS,ABSERR,ALIST,BLIST,CHEBMO,CORREC,CYCLE,C1,C2,DL,
     *  DLA,
     *  DRL,ELIST,EP,EPS,EPSA,EPSABS,ERLST,ERRSUM,F,FACT,OMEGA,P,PI,P1,
     *  PSUM,RESEPS,RESULT,RES3LA,RLIST,RSLST,UFLOW,EPMACH,OFLOW
      INTEGER IER,IERLST,INTEGR,IORD,KTMIN,L, LST,LIMIT,LIMLST,LL,
     *  MAXP1,MOMCOM,NEV,NEVAL,NNLOG,NRES,NUMRL2
C
      DIMENSION ALIST(LIMIT),BLIST(LIMIT),CHEBMO(MAXP1,25),ELIST(LIMIT),
     *  ERLST(LIMLST),IERLST(LIMLST),IORD(LIMIT),NNLOG(LIMIT),PSUM(52),
     *  RES3LA(3),RLIST(LIMIT),RSLST(LIMLST)
C
      EXTERNAL F
C
C
C           THE DIMENSION OF PSUM IS DETERMINED BY THE VALUE OF
C           LIMEXP IN SUBROUTINE QEXT (PSUM MUST BE
C           OF DIMENSION (LIMEXP+2) AT LEAST).
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           C1, C2 - END POINT OF SUBINTERVAL (OF LENGTH CYCLE)
C           CYCLE  - (2*INT(ABS(OMEGA))+1)*PI/ABS(OMEGA)
C           PSUM   - VECTOR OF DIMENSION AT LEAST (LIMEXP+2)
C                    (SEE ROUTINE QEXT)
C                    PSUM CONTAINS THE PART OF THE EPSILON TABLE
C                    WHICH IS STILL NEEDED FOR FURTHER COMPUTATIONS.
C                    EACH ELEMENT OF PSUM IS A PARTIAL SUM OF
C                    THE SERIES WHICH SHOULD SUM TO THE VALUE OF
C                    THE INTEGRAL.
C           ERRSUM - SUM OF ERROR ESTIMATES OVER THE SUBINTERVALS,
C                    CALCULATED CUMULATIVELY
C           EPSA   - ABSOLUTE TOLERANCE REQUESTED OVER CURRENT
C                    SUBINTERVAL
C           CHEBMO - ARRAY CONTAINING THE MODIFIED CHEBYSHEV MOMENTS
C                    (SE ALSO ROUTINE QC25F)
C
      DATA P/9.0D-01/,PI/3.1415926535897932D+00/
C
C           MACHINE DEPENDENT COSTANTS
C           --------------------------
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C           TEST VALIDITY OF PARAMETERS
C           ---------------------------
C
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      NEVAL = 0
      LST = 0
      IER = 0
      IF((INTEGR.NE.1.AND.INTEGR.NE.2).OR.EPSABS.LE.0.0D+00.OR.
     *  LIMLST.LT.3) IER = 6
      IF(IER.EQ.6) GO TO 999
      IF(OMEGA.NE.0.0D+00) GO TO 10
C
C           INTEGRATION BY QAGI IF OMEGA IS ZERO
C           ------------------------------------
C
      IF(INTEGR.EQ.1) CALL QAGI(F,0.0D+00,1,EPSABS,0.0D+00,
     *  RESULT,ABSERR,NEVAL,IER)
      RSLST(1) = RESULT
      ERLST(1) = ABSERR
      IERLST(1) = IER
      LST = 1
      GO TO 999
C
C           INITIALIZATIONS
C           ---------------
C
   10 L = DABS(OMEGA)
      DL = 2*L+1
      CYCLE = DL*PI/DABS(OMEGA)
      IER = 0
      KTMIN = 0
      NEVAL = 0
      NUMRL2 = 0
      NRES = 0
      C1 = A
      C2 = CYCLE+A
      P1 = 1.0D+00-P
      EPS = EPSABS
      IF(EPSABS.GT.UFLOW/P1) EPS = EPSABS*P1
      EP = EPS
      FACT = 1.0D+00
      CORREC = 0.0D+00
      ABSERR = 0.0D+00
      ERRSUM = 0.0D+00
C
C           MAIN DO-LOOP
C           ------------
C
      DO 50 LST = 1,LIMLST
C
C           INTEGRATE OVER CURRENT SUBINTERVAL.
C
      DLA = LST
      EPSA = EPS*FACT
      CALL QFOUR(F,C1,C2,OMEGA,INTEGR,EPSA,0.0D+00,LIMIT,LST,MAXP1,
     *  RSLST(LST),ERLST(LST),NEV,IERLST(LST),ALIST,BLIST,RLIST,ELIST,
     *  IORD,NNLOG,MOMCOM,CHEBMO)
      NEVAL = NEVAL+NEV
      FACT = FACT*P
      ERRSUM = ERRSUM+ERLST(LST)
      DRL = 5.0D+01*DABS(RSLST(LST))
C
C           TEST ON ACCURACY WITH PARTIAL SUM
C
      IF((ERRSUM+DRL).LE.EPSABS.AND.LST.GE.6) GO TO 80
      CORREC = DMAX1(CORREC,ERLST(LST))
      IF(IERLST(LST).NE.0) EPS = DMAX1(EP,CORREC*P1)
      IF(IERLST(LST).NE.0) IER = 7
      IF(IER.EQ.7.AND.(ERRSUM+DRL).LE.CORREC*1.0D+01.AND.
     *  LST.GT.5) GO TO 80
      NUMRL2 = NUMRL2+1
      IF(LST.GT.1) GO TO 20
      PSUM(1) = RSLST(1)
      GO TO 40
   20 PSUM(NUMRL2) = PSUM(LL)+RSLST(LST)
      IF(LST.EQ.2) GO TO 40
C
C           TEST ON MAXIMUM NUMBER OF SUBINTERVALS
C
        IF(LST.EQ.LIMLST) IER = 8
C
C           PERFORM NEW EXTRPOLATION
C
        CALL QEXT(NUMRL2,PSUM,RESEPS,ABSEPS,RES3LA,NRES)
C
C           TEST WHETHER EXTRAPOLATED RESULT IS INFLUENCED BY ROUNDOFF
C
         KTMIN = KTMIN+1
         IF(KTMIN.GE.15.AND.ABSERR.LE.1.0D-03*(ERRSUM+DRL)) IER = 9
         IF(ABSEPS.GT.ABSERR.AND.LST.NE.3) GO TO 30
         ABSERR = ABSEPS
         RESULT = RESEPS
         KTMIN = 0
C
C           IF IER.NE.0, CHECK WHETHER DIRECT RESULT (PARTIAL SUM) OR
C           EXTRAPOLATED RESULT YIELDS THE BEST INTEGRAL APPROXIMATION
C
        IF((ABSERR+1.0D+01*CORREC).LE.EPSABS.OR.
     *  (ABSERR.LE.EPSABS.AND.1.0D+01*CORREC.GE.EPSABS)) GO TO 60
   30   IF(IER.NE.0.AND.IER.NE.7) GO TO 60
   40   LL = NUMRL2
        C1 = C2
        C2 = C2+CYCLE
   50 CONTINUE
C
C           SET FINAL RESULT AND ERROR ESTIMATE
C           -----------------------------------
C
   60 ABSERR = ABSERR+1.0D+01*CORREC
      IF(IER.EQ.0) GO TO 999
      IF(RESULT.NE.0.0D+00.AND.PSUM(NUMRL2).NE.0.0D+00) GO TO 70
      IF(ABSERR.GT.ERRSUM) GO TO 80
      IF(PSUM(NUMRL2).EQ.0.0D+00) GO TO 999
   70 IF(ABSERR/DABS(RESULT).GT.(ERRSUM+DRL)/DABS(PSUM(NUMRL2)))
     * GO TO 80
      IF(IER.GE.1.AND.IER.NE.7) ABSERR = ABSERR+DRL
      GO TO 999
   80 RESULT = PSUM(NUMRL2)
      ABSERR = ERRSUM+DRL
  999 RETURN
      END

















