$DEBUG
      SUBROUTINE QAWO(F,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER)
C
C.....................................................................
C
C 1.     QAWO
C        COMPUTATION OF OSCILLATORY INTEGRALS
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN   C
C           DEFINITE INTEGRAL
C                I = INTEGRAL OF F(X)*W(X) OVER (A,INFINITY)
C                    WHERE W(X) = COS(OMEGA*X)
C                       OR W(X) = SIN(OMEGA*X),
C           HOPEFULLY SATISFAYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.     CALLING SEQUENCE
C           CALL QAWO(F,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
C                     NEVAL,IER)
C
C        PARAMETERS
C         ON ENTRY
C           F      -REAL
C                   FUNCTION SUBPROGRAM DEFINING THE
C                   FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO
C                   BE DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      -REAL
C                   LOWER LIMIT OF INTEGRATION
C
C           B      -REAL
C                   UPPER LIMIT OF INTEGRATION
C
C           OMEGA  -REAL
C                   PARAMETER IN THE WEIGHT FUNCTION
C
C           INTEGR -INTEGER
C                   INDICATES WHICH OF THE WEIGHT FUNCTIONS IS USED
C                      INTEGR = 1      W(X) = COS(OMEGA*X)
C                      INTEGR = 2      W(X) = SIN(OMEGA*X)
C                   IF INTEGR.NE.1.AND.INTEGR.NE.2, THE ROUTINE
C                   WILL END WITH IER = 6.
C
C           EPSABS -REAL
C                   ABSOLUTE ACCURACY REQUESTED
C           EPSREL -REAL
C                   RELATIVE ACCURACY REQUESTED
C                   IF EPSABS.LT.0 AND EPSREL.LT.0,
C                   THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICHSHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE.
C                            THE ESTIMATES FOR INTEGRAL AND ERROR ARE
C                            LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            (= LIMIT) HAS BEEN ACHIEVED. ONE CAN
C                            ALLOW MORE SUBDIVISIONS BY INCREASING THE
C                            VALUE OF LIMIT (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENTS INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT IT
C                            IS ADVISED TO ANALYZE THE INTEGRAND IN
C                            ORDER TO DETERMINE THE INTEGRATION
C                            DIFFICULTIES. IF THE POSITION OF A LOCAL
C                            DIFFICULTY CAN BE DETERMINED (E.G.
C                            SINGULARITY, DISCONTINUITY WITHIN THE
C                            INTERVAL) ONE WILL PROBABLY GAIN FROM
C                            SPLITTING UP THE INTERVAL AT THIS POINT
C                            AND CALLING THE INTEGRATOR ON THE
C                            SUBRANGES. IF POSIBLE, AN APPROPRIATE
C                            SPECIAL-PURPOSE INTEGRATOR SHOULD
C                            BE USED WHICH IS DESIGNED FOR HANDLING
C                            THE TYPE OF DIFFICULTY INVOLVED.
C                        = 2 THE OCCURENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                            THE ERROR MAY BE UNDER ESTIMATED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME INTERIOR POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 4 THE ALGORITHM DOES NOT CONVERGE. ROUNDOFF
C                            ERROR IS DETECTED IN THE EXTRAPOLATION
C                            TABLE. IT IS PRESUMED THAT THE REQUESTED
C                            TOLERANCE CANNOT BE ACHIEVED DUE TO
C                            ROUNDOFF IN THE EXTRAPOLATION TABLE, AND
C                            THAT THE RETURNED RESULT IS THE BEST WHICH
C                            CAN BE OBTAINED.
C                        = 5 THE INTEGRAL IS PROBABLY DIVERGENT, OR
C                            SLOWLY CONVERGENT. IT MUST BE NOTED THAT
C                            DIVERGENCE CAN OCCUR WITH ANY OTHER VALUE
C                            OF IER.GT.0.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR, NEVAL ARE SET TO ZERO.
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C             - QFOUR
C             - QEXT
C             - QSORT
C             - QC25O
C             - QK15W
C             - QCHEB
C             - QWGTO
C             - QMACO
C             - FORTRAN ABS, AMAX1, AMIN1
C             - F (USER-PROVIDED FUNCTION)
C
C.....................................................................
C
      REAL*8 A,ABSERR,ALIST,B,BLIST,CHEBMO,ELIST,EPSABS,EPSREL,F,OMEGA,
     *  RESULT,RLIST
      INTEGER IER,INTEGR,IORD,LIMIT,MAXP1,MOMCOM,NEVAL,NNLOG
C
      DIMENSION ALIST(500),BLIST(500),CHEBMO(21,25),ELIST(500),
     *  IORD(500),NNLOG(500),RLIST(500)
C
      EXTERNAL F
C
C        LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN THE
C        SUBDIVISION PROCES OF QFOUR. TAKE CARE THAT LIMIT.GE.1.
C
      DATA LIMIT/500/
C
C        MAXP1 GIVES AN UPPER BOUND ON THE NUMBER OF CHEBYSHEV MOMENTS
C        WHICH CAN BE STORED, I.E. FOR THE INTERVALS OF LENGTHS
C        ABS(B-A)*2**(-L), L = 0, 1, ... , MAXP1-2. TAKE CARE THAT
C        MAXP1.GE.1.
C
      DATA MAXP1/21/
C
C***FIRST EXECUTABLE STATEMENT
      CALL QFOUR(F,A,B,OMEGA,INTEGR,EPSABS,EPSREL,LIMIT,1,MAXP1,RESULT,
     *    ABSERR,NEVAL,IER,ALIST,BLIST,RLIST,ELIST,IORD,NNLOG,MOMCOM,
     *    CHEBMO)
      RETURN
      END
