      SUBROUTINE QAWS(F,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,
     *  ABSERR,NEVAL,IER)
C
C......................................................................
C
C 1.     QAWS
C        INTEGRATION OF FUNCTIONS HAVING ALGEBRAICO-LOGARITHMIC
C        END POINT SINGULARITIES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C           DEFINING INTEGRAL  I= INTEGRAL OF F*W OVER (A,B) (WHERE W
C           SHOWS A SINGULAR BEHAVIOUR AT THE END POINTS, SEE PARAMETER
C           INTEGR), HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.     CALLING SEQUENCE
C           CALL QAWS(F,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,
C                     ABSERR,NEVAL,IER)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUNCTION SUBPROGRAM DEFINIG THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION, B.GT.A
C                    IF B.LE.A, THE ROUTINE WILL END WITH IER = 6.
C
C           ALFA   - REAL
C                    PARAMETER IN THE INTEGRAND FUNCTION, ALFA.GT.(-1)
C                    IF ALFA.LE.(-1), THE ROUTINE WILL END WITH IER = 6.
C
C           BETA   - REAL
C                    PARAMETER IN THE INTEGRAND FUNCTION, BETA.GT.(-1)
C                    IF BETA.LE.(-1), THE ROUTINE WILL END WITH IER = 6.
C
C           INTEGR - INTEGER
C                    INDICATES WHICH WEIGHT FUNCTION IS TO BE USED
C                    = 1  (X-A)**ALFA*(B-X)**BETA
C                    = 2  (X-A)**ALFA*(B-X)**BETA*LOG(X-A)
C                    = 3  (X-A)**ALFA*(B-X)**BETA*LOG(B-X)
C                    = 4  (X-A)**ALFA*(B-X)**BETA*LOG(X-A)*LOG(B-X)
C                    IF INTEGR.LT.1 OR INTEGR.GT.4, THE ROUTINE WILL
C                    END WITH IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR THE INTEGRAL AND ERROR
C                            ARE LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                    IER = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE DATA VALUE
C                            OF LIMIT IN QAWS (AND TAKING THE ACCORDING
C                            DIMENSION ADJUSTMENTS INTO ACCOUNT).
C                            HOWEVER, IF THIS YIELDS NO IMPROVEMENT IT
C                            IS ADVISED TO ANALYZE THE INTEGRAND, IN
C                            ORDER TO DETERMINE THE INTEGRATION
C                            DIFFICULTIES WHICH PREVENT THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED. IN CASE OF
C                            A JUMP DISCONTINUITY OR A LOCAL
C                            SINGULARITY OF ALGEBRAICO-LOGARITHMIC TYPE
C                            AT ONE MORE INTERIOR POINTS OF THE
C                            INTEGRATION RANGE, ONE SHOULD PROCEED BY
C                            SPLITTING UP THE INTERVAL AT THESE POINTS
C                            AND CALLING THE INTEGRATOR ON THE
C                            SUBRANGES.
C                        = 2 THE OCCURRENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            B.LE.A OR ALFA.LE.(-1) OR BETA .LE.(-1)
C                            OR INTEGR.LT.1 OR INTEGR.GT.4 OR
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            RESULT, ABSERR, NEVAL ARE SET TO ZERO.
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C              - QAWSE
C              - QMOMO
C              - QC25S
C              - QSORT
C              - QK15W
C              - QWGTS
C              - QCHEB
C              - F (USER-PROVIDED FUNCTION)
C              - QMACO
C              - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,ABSERR,ALFA,ALIST,B,BLIST,BETA,ELIST,EPSABS,EPSREL,F,
     *  RESULT,RLIST
      INTEGER IER,INTEGR,IORD,LAST,LIMIT,NEVAL
C
      DIMENSION ALIST(500),BLIST(500),ELIST(500),IORD(500),RLIST(500)
      EXTERNAL F
C
C         LIMIT IS THE MAXIMUM NUMBER OF SUBINTERVALS ALLOWED IN THE
C         SUBDIVISION PROCESS OF QAWSE. TAKE CARE THAT LIMIT.GE.2.
C
      DATA LIMIT/500/
C
C***FIRST EXECUTABLE STATEMENT
C
      CALL QAWSE(F,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,LIMIT,RESULT,
     *  ABSERR,NEVAL,IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
      RETURN
      END


