      SUBROUTINE QAWSE(F,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,LIMIT,
     *  RESULT,ABSERR,NEVAL,IER,ALIST,BLIST,RLIST,ELIST,IORD,LAST)
C
C......................................................................
C
C 1.     QAWSE
C        INTEGRATION OF FUNCTIONS HAVING ALGEBRAICO-LOGARITHMIC
C        END POINT SINGULARITIES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO A GIVEN
C           DEFINITE INTEGRAL  I = INTEGRAL OF F*W OVER (A,B), (WHERE W
C           SHOWS A SINGULAR BEHAVIOUR AT THE END POINTS, SEE PARAMETER
C           INTEGR), HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C           ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C 3.     CALLING SEQUENCE
C           CALL QAWSE(F,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,LIMIT,
C                      RESULT,ABSERR,NEVAL,IER,ALIST,BLIST,RLIST,ELIST,
C                      IORD,LAST)
C
C        PARAMETERS
C         ON ENTRY
C           F      - REAL
C                    FUUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE DRIVER PROGRAM.
C
C           A      - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C           B      - REAL
C                    UPPER LIMIT OF INTEGRATION, B.GT.A
C                    IF B.LE.A, THE ROUTINE WILL END WITH IER = 6.
C
C           ALFA   - REAL
C                    PARAMETER IN THE WEIGHT FUNCTION, ALFA.GT.(-1)
C                    IF ALFA.LE.(-1), THE ROUTINE WILL END WITH
C                    IER = 6.
C
C           BETA   - REAL
C                    PARAMETER IN THE WEIGHT FUNCTION, BETA.GT.(-1)
C                    IF BETA.LE.(-1), THE ROUTINE WILL END WITH
C                    IER = 6.
C
C           INTEGR - INTEGER
C                    INDICATES WHICH WEIGHT FUNCTIONIS TO BE USED
C                    = 1  (X-A)**ALFA*(B-X)**BETA
C                    = 2  (X-A)**ALFA*(B-X)**BETA*LOG(X-A)
C                    = 3  (X-A)**ALFA*(B-X)**BETA*LOG(B-X)
C                    = 4  (X-A)**ALFA*(B-X)**BETA*LOG(X-A)*LOG(B-X)
C                    IF INTEGR.LT.1 OR INTEGR.GT.4, THE ROUTINE WILL
C                    END WITH IER = 6.
C
C           EPSABS - REAL
C                    ABSOLUTE ACCURACY REQUESTED
C           EPSREL - REAL
C                    RELATIVE ACCURACY REQUESTED
C                    IF EPSABS.LT.0 AND EPSREL.LT.0,
C                    THE ROUTINE WILL END WITH IER = 6.
C
C           LIMIT  - INTEGER
C                    GIVES AN UPPER BOUND ON THE NUMBER OF SUBINTERVALS
C                    IN THE PARTITION OF (A,B), LIMIT.GE.2
C                    IF LIMIT.LT.2, THE ROUTINE WILL END WITH IER = 6.
C
C         ON RETURN
C           RESULT - REAL
C                    APPROXIMATION TO THE INTEGRAL
C
C           ABSERR - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  - INTEGER
C                    NUMBER OF INTEGRAND EVALUATIONS
C
C           IER    - INTEGER
C                    IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                            ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                            ACCURACY HAS BEEN ACHIEVED.
C                    IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE
C                            THE ESTIMATES FOR THE INTEGRAL AND ERROR
C                            ARE LESS RELIABLE. IT IS ASSUMED THAT THE
C                            REQUESTED ACCURACY HAS NOT BEEN ACHIEVED.
C                        = 1 MAXIMUM NUMBER OF SUBDIVISIONS ALLOWED
C                            HAS BEEN ACHIEVED. ONE CAN ALLOW MORE
C                            SUBDIVISIONS BY INCREASING THE VALUE OF
C                            LIMIT. HOWEVER, IF THIS YIELDS NO
C                            IMPROVEMENT IT IS ADVISED TO ANALYZE THE
C                            INTEGRAND, IN ORDER TO DETERMINE THE
C                            INTEGRATION DIFFICULTIES WHICH PREVENT
C                            THE REQUESTED TOLERANCE FROM BEING
C                            ACHIEVED. IN CASE OF A JUMP DISCONTINUITY
C                            OR A LOCAL SINGULARITY OF ALGEBRAICO-
C                            LOGARITHMIC TYPE AT ONE OR MORE INTERIOR
C                            POINTS OF THE INTEGRATION RANGE, ONE
C                            SHOULD PROCEED BY SPLLITING UP THE
C                            INTERVAL AT THESE POINTS AND CALLING THE
C                            INTEGRATOR ON THE SUBRANGES.
C                        = 2 THE OCCURENCE OF ROUNDOFF ERROR IS
C                            DETECTED, WHICH PREVENTS THE REQUESTED
C                            TOLERANCE FROM BEING ACHIEVED.
C                        = 3 EXTREMELY BAD INTEGRAND BEHAVIOUR OCCURS
C                            AT SOME POINTS OF THE INTEGRATION
C                            INTERVAL.
C                        = 6 THE INPUT IS INVALID, BECAUSE
C                            B.LE.A OR ALFA.LE.(-1) OR BETA.LE(-1) OR
C                            INTEGR.LT.1 OR INTEGR.GT.4, OR
C                            EPSABS.LT.0 AND EPSREL.LT.0,
C                            OR LIMIT.LT.2.
C                            RESULT, ABSERR, NEVAL, RLIST(1), ELIST(1),
C                            IORD(1) AND LAST ARE SET TO ZERO.
C                            ALIST(1) AND BLIST(1) ARE SET TO A AND B
C                            RESPECTIVELY.
C
C           ALIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                     LAST ELEMENTS OF WHICH ARE THE LEFT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B)
C
C           BLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                     LAST ELEMENTS OF THE WHICH ARE THE RIGHT END POINTS
C                    OF THE SUBINTERVALS IN THE PARTITION OF THE GIVEN
C                    INTEGRATION RANGE (A,B)
C
C           RLIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                     LAST ELEMENTS OF WHICH ARE THE INTEGRAL
C                    APPROXIMATIONS ON THE SUBINTERVALS
C
C           ELIST  - REAL
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST
C                     LAST ELEMENTS OF WHICH ARE THE MODULI OF THE
C                    ABSOLUTE ERROR ESTIMATES ON THE SUBINTERVALS
C
C           IORD   - INTEGER
C                    VECTOR OF DIMENSION AT LEAST LIMIT, THE FIRST K
C                    ELEMENTS OF WHICH ARE POINTERS TO THE ERROR
C                    ESTIMATES OVER THE SUBINTERVALS, SO THAT
C                    ELIST(IORD(1)), ..., ELIST(IORD(K)) WITH K = LAST
C                    IF LAST.LE.(LIMIT/2+2), AND K = LIMIT+1-LAST
C                    OTHERWISE, FORM A DECREASING SEQUENCE.
C
C           LAST   - INTEGER
C                    NUMBER OF SUBINTERVALS ACTUALLY PRODUCED IN THE
C                    SUBDIVISION PROCESS
C
C 4.     SUBROUTINES OR FUNCTIOS NEEDED
C             - QMOMO
C             - QC25S
C             - QSORT
C             - QK15W
C             - QCHEB
C             - QMACO
C             - QWGTS
C             - F (USER-PROVIDED FUNCTION)
C             - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,ABSERR,ALFA,ALIST,AREA,AREA1,AREA12,AREA2,A1,A2,B,BETA,
     *  BLIST,B1,B2,CENTRE,ELIST,EPMACH,EPSABS,EPSREL,ERRBND,ERRMAX,
     *  ERROR1,ERRO12,ERROR2,ERRSUM,F,OFLOW,RESAS1,RESAS2,RESULT,
     *  RG,RH,RI,RJ,RLIST,UFLOW
      INTEGER IER,INTEGR,IORD,IROFF1,IROFF2,K,LAST,LIMIT,MAXERR,NEV,
     *  NEVAL,NRMAX
C
      EXTERNAL F
C
      DIMENSION ALIST(LIMIT),BLIST(LIMIT),RLIST(LIMIT),ELIST(LIMIT),
     *  IORD(LIMIT),RI(25),RJ(25),RH(25),RG(25)
C
C            LIST OF MAJOR VARIABLES
C            -----------------------
C
C           ALIST    - LIST OF LEFT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           BLIST    - LIST OF RIGHT END POINTS OF ALL SUBINTERVALS
C                      CONSIDERED UP TO NOW
C           RLIST(I) - APPROXIMATION TO THE INTEGRAL OVER
C                      (ALIST(I),BLIST(I))
C           ELIST(I) - ERROR ESTIMATE APPLYING TO RLIST(I)
C           MAXERR   - POINTER TO THE INTERVAL WITH LARGEST ERROR
C                      ESTIMATE
C           ERRMAX   - ELIST(MAXERR)
C           AREA     - SUM OF THE INTEGRALS OVER THE SUBINTERVALS
C           ERRSUM   - SUM OF THE ERRORS OVER THE SUBINTERVALS
C           ERRBND   - REQUESTED ACCURACY MAX(EPSABS,EPSREL*
C                      ABS(RESULT))
C           *****1   - VARIABLE FOR THE LEFT SUBINTERVAL
C           *****2   - VARIABLE FOR THE RIGHT SUBINTERVAL
C           LAST     - INDEX FOR SUBDIVISION
C
C
C            MACHINE DEPENDENT CONSTANTS
C            ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C           TEST OF VALIDITY OF PARAMETERS
C           ------------------------------
C
      IER = 6
      NEVAL = 0
      LAST = 0
      RLIST(1) = 0.0D+00
      ELIST(1) = 0.0D+00
      IORD(1) = 0
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      IF(B.LE.A.OR.(EPSABS.LT.0.0D+00.AND.EPSREL.LT.0.0D+00).OR.
     *  ALFA.LE.(-1.0D+00).OR.BETA.LE.(-1.0D+00).OR.INTEGR.LT.1.
     *  OR.INTEGR.GT.4.OR.LIMIT.LT.2) GO TO 999
      IER = 0
C
C           COMPUTE THE MODIFIED CHEBYSHEV MOMENTS.
C
      CALL QMOMO(ALFA,BETA,RI,RJ,RG,RH,INTEGR)
C
C           INTEGRATE OVER THE INTERVALS (A,(A+B)/2) AND ((A+B)/2,B).
C
      CENTRE = 5.0D-01*(B+A)
      CALL QC25S(F,A,B,A,CENTRE,ALFA,BETA,RI,RJ,RG,RH,AREA1,ERROR1,
     *  RESAS1,INTGR,NEV)
      NEVAL = NEV
      CALL QC25S(F,A,B,CENTRE,B,ALFA,BETA,RI,RJ,RG,RH,AREA2,ERROR2,
     *  RESAS2,INTEGR,NEV)
      LAST = 2
      NEVAL = NEVAL+NEV
      RESULT = AREA1+AREA2
      ABSERR = ERROR1+ERROR2
C
C           TEST ON ACCURACY.
C
      ERRBND = DMAX1(EPSABS,EPSREL*DABS(RESULT))
C
C           INITIALIZATION
C           --------------
C
      IF(ERROR2.GT.ERROR1) GO TO 10
      ALIST(1) = A
      ALIST(2) = CENTRE
      BLIST(1) = CENTRE
      BLIST(2) = B
      RLIST(1) = AREA1
      RLIST(2) = AREA2
      ELIST(1) = ERROR1
      ELIST(2) = ERROR2
      GO TO 20
   10 ALIST(1) = CENTRE
      ALIST(2) = A
      BLIST(1) = B
      BLIST(2) = CENTRE
      RLIST(1) = AREA2
      RLIST(2) = AREA1
      ELIST(1) = ERROR2
      ELIST(2) = ERROR1
   20 IORD(1) = 1
      IORD(2) = 2
      IF(LIMIT.EQ.2) IER = 1
      IF(ABSERR.LE.ERRBND.OR.IER.EQ.1) GO TO 999
      ERRMAX = ELIST(1)
      MAXERR = 1
      NRMAX = 1
      AREA = RESULT
      ERRSUM = ABSERR
      IROFF1 = 0
      IROFF2 = 0
C
C           MAIN DO-LOOP
C           ------------
C
      DO 60 LAST = 3,LIMIT
C
C           BISECT THE SUBINTERVAL WITH LARGEST ERROR ESTIMATE.
C
        A1 = ALIST(MAXERR)
        B1 = 5.0D-01*(ALIST(MAXERR)+BLIST(MAXERR))
        A2 = B1
        B2 = BLIST(MAXERR)
C
        CALL QC25S(F,A,B,A1,B1,ALFA,BETA,RI,RJ,RG,RH,AREA1,ERROR1,
     *  RESAS1,INTEGR,NEV)
        NEVAL = NEVAL+NEV
        CALL QC25S(F,A,B,A2,B2,ALFA,BETA,RI,RJ,RG,RH,AREA2,ERROR2,
     *  RESAS2,INTEGR,NEV)
        NEVAL = NEVAL+NEV
C
C           IMPROVE PREVIOUS APPROXIMATIONS INTEGRAL AND ERROR AND
C           TEST FOR ACCURACY.
C
        AREA12 = AREA1+AREA2
        ERRO12 = ERROR1+ERROR2
        ERRSUM = ERRSUM+ERRO12-ERRMAX
        AREA = AREA+AREA12-RLIST(MAXERR)
        IF(A.EQ.A1.OR.B.EQ.B2) GO TO 30
        IF(RESAS1.EQ.ERROR1.OR.RESAS2.EQ.ERROR2) GO TO 30
C
C           TEST FOR ROUNDOFF ERROR.
C
        IF(DABS(RLIST(MAXERR)-AREA12).LT.1.0D-05*DABS(AREA12)
     *  .AND.ERRO12.GE.9.9D-01*ERRMAX) IROFF1 = IROFF+1
        IF(LAST.GT.10.AND.ERRO12.GT.ERRMAX) IROFF2 = IROFF2+1
   30   RLIST(MAXERR) = AREA1
        RLIST(LAST) = AREA2
C
C           TEST ON ACCURACY.
C
        ERRBND = DMAX1(EPSABS,EPSREL*DABS(AREA))
        IF(ERRSUM.LE.ERRBND) GO TO 35
C
C           SET ERROR FLAG IN THE CASE THAT THE NUMBER OF INTERVAL
C           BISECTIONS EXCEEDS LIMIT.
C
        IF(LAST.EQ.LIMIT) IER = 1
C
C           SET ERROR FLAG IN THE CASE OF ROUNDOFF ERROR.
C
        IF(IROFF1.GE.6.OR.IROFF2.GE.20) IER = 2
C
C           SET ERROR FLAG IN THE CASE OF BAD INTEGRAND BEHAVIOUR
C           AT INTERIOR POIMTS OF INTEGRATION RANGE.
C
        IF(DMAX1(DABS(A1),DABS(B2)).LE.(1.0D+00+1.0D+03*EPMACH)*
     *  (DABS(A2)+1.0D+03*UFLOW)) IER = 3
C
C           APPEND THE NEWLY-CREATED INTERVALS TO THE LIST.
C
   35   IF(ERROR2.GT.ERROR1) GO TO 40
        ALIST(LAST) = A2
        BLIST(MAXERR) = B1
        BLIST(LAST) = B2
        ELIST(MAXERR) = ERROR1
        ELIST(LAST) = ERROR2
        GO TO 50
   40   ALIST(MAXERR) = A2
        ALIST(LAST) = A1
        BLIST(LAST) = B1
        RLIST(MAXERR) = AREA2
        RLIST(LAST) = AREA1
        ELIST(MAXERR) = ERROR2
        ELIST(LAST) = ERROR1
C
C           CALL SUBROUTINE QSORT TO MAINTAIN THE DESCENDING ORDERING
C           IN THE LIST OF ERROR ESTIMATES AND SELECT THE SUBINTERVAL
C           WITH LARGEST ERROR ESTIMATE (TO BISEXTED NEXT).
C
   50   CALL QSORT(LIMIT,LAST,MAXERR,ERRMAX,ELIST,IORD,NRMAX)
C***JUMP OUT OF DO-LOOP
        IF(IER.NE.0.OR.ERRSUM.LE.ERRBND) GO TO 70
   60 CONTINUE
C
C           COMPUTE FINAL RESULT.
C           ---------------------
C
   70 RESULT = 0.0D+00
      DO 80 K=1,LAST
        RESULT = RESULT+RLIST(K)
   80 CONTINUE
      ABSERR = ERRSUM
  999 RETURN
      END
