      SUBROUTINE QC25S(F,A,B,BL,BR,ALFA,BETA,RI,RJ,RG,RH,RESULT,ABSERR,
     *  RESASC,INTEGR,NEV)
C
C......................................................................
C
C 1.     QC25S
C        INTEGRATION RULES FOR INTEGRANDS HAVING ALGEBRAICO-LOGARITHMIC
C        END POINT SINGULARITIES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE I = INTEGRAL OF F*W OVER (BL,BR), WITH ERROR
C           ESTIMATE, WHERE THE WEIGHT FUNCTION W HAS A SINGULAR
C           BEHAVIOUR OF ALGEBRAICO-LOGARITHMIC TYPE AT THE POINTS
C           A AND/OR B. (BL,BR) IS A PART OF (A,B).
C
C 3.     CALLING SEQUENCE
C           CALL QC25S(F,A,B,BL,BR,ALFA,BETA,RI,RJ,RG,RH,RESULT,ABSERR,
C                      RESASC,INTEGR,NEV)
C
C        PARAMETERS
C           F      -REAL
C                   FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                   F(X). THE ACTUAL NAME FOR F NEEDS TO BE DECLARED
C                   E X T E R N A L  IN THE DRIVER PROGRAM.
C
C           A      -REAL
C                   LEFT END POINT OF THE ORIGINAL INTERVAL
C
C           B      -REAL
C                   RIGHT END POINT OF THE ORIGINAL INTERVAL, B.GT.A
C
C           BL     -REAL
C                   LOWER LIMIT OF INTEGRATION, BL.GE.A
C
C           BR     -REAL
C                   UPPER LIMIT OF INTEGRATION, BR.LE.B
C
C           ALFA   -REAL
C                   PARAMETER IN THE WEIGHT FUNCTION
C
C           BETA   -REAL
C                   PARAMETER IN THE WEIGHT FUNCTION
C
C           RI,RJ,RG,RH, -REAL
C                   MODIFIED CHEBYSHEV MOMENTS FOR THE APPLICATION
C                   OF THE GENERALIZED CLENSHAW-CURTIS METHOD
C                   (COMPUTED IN SUBROUTINE QMOMO)
C
C           RESULT -REAL
C                   APPROXIMATION TO THE INTEGRAL
C                   RESULT IS COMPUTED BY USING A GENERALIZED
C                   CLENSHAW-CURTIS METHOD IF B1 = A OR BR = B.
C                   IN ALL OTHER CASES THE 15-POINT KRONROD RULE IS
C                   APPLIED, OBTAINED BY OPTIMAL ADDITION OF ABSCISSAE
C                   TO THE 7-POINT GAUSS RULE.
C
C           ABSERR -REAL
C                   ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                   WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           RESASC -REAL
C                   APPROXIMATION TO THE INTEGRAL OF ABS(F*W-I/(B-A))
C
C           INTEGR -INTEGER
C                   WHICH DETERMINES THE WEIGHT FUNCTION
C                   = 1  W(X) = (X-A)**ALFA*(B-X)**BETA
C                   = 2  W(X) = (X-A)**ALFA*(B-X)**BETA*LOG(X-A)
C                   = 3  W(X) = (X-A)**ALFA*(B-X)**BETA*LOG(B-X)
C                   = 4  W(X) = (X-A)**ALFA*(B-X)**BETA*LOG(X-A)*
C                               LOG(B-X)
C
C           NEV    -INTEGER
C                   NUMBER OF INTEGRAND EVALUATIONS
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C             - QCHEB
C             - QK15W
C             - F (USER-PROVIDED FUNCTION)
C             - QWGTS
C             - FORTRAN ABS, ALOG, AMAX1
C
C......................................................................
C
      REAL*8 A,ABSERR,ALFA,B,BETA,BL,BR,CENTR,CHEB12,CHEB24,DC,F,FACTOR,
     *  FIX,FVAL,HLGTH,QWGTS,RESABS,RESASC,RESULT,RES12,RES24,RG,
     *  RH,RI,RJ,U,X
      INTEGER I,INTEGR,ISYM,NEV
C
      DIMENSION CHEB12(13),CHEB24(25),FVAL(25),RG(25),RH(25),RI(25),
     *  RJ(25),X(11)
C
      EXTERNAL F,QWGTS
C
C           THE VECTOR X CONTAINS THE VALUES COS(K*PI/24)
C           K = 1, ...,11 , TO BE USED FOR THE COMPUTATION OF THE
C           CHEBYSHEV SERIES EXPANSION OF F.
C
      DATA X(1),X(2),X(3),X(4),X(5),X(6),X(7),X(8),X(9),X(10),X(11)/
     *     9.914448613738104D-01,        9.659258262890683D-01,
     *     9.238795325112868D-01,        8.660254037844386D-01,
     *     7.933533402912352D-01,        7.071067811865475D-01,
     *     6.087614290087206D-01,        5.000000000000000D-01,
     *     3.826834323650898D-01,        2.588190451025208D-01,
     *     1.305261922200516D-01/
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           FVAL   -VALUE OF THE FUNCTION F AT THE POINTS
C                   (BR-BL)*0.5*COS(K*PI/24)+(BR+BL)*0.5
C                   K = 0, ..., 24
C           CHEB12 -COEFFICIENTS OF THE CHEBYSHEV SERIES EXPANSION
C                   OF DEGREE 12, FOR THE FUNCTION F, IN THE INTERVAL
C                   (BL,BR)
C           CHEB24 -COEFFICIENTS OF THE CHEBYSHEV SERIES EXPANSION
C                   OF DEGREE 24, FOR THE FUNCTION F, IN THE INTERVAL
C                   (BL,BR)
C           RES12  -APPROXIMATION TO THE INTEGRAL OBTAINED FROM CHEB12
C           RES24  -APPROXIMATION TO THE INTEGRAL OBTAINED FROM CHEB24
C           QWGTS  -EXTERNAL FUNCTION SUBPROGRAM DEFINING THE FOUR
C                   POSSIBLE WEIGHT FUNCTIONS
C           HLGTH  -HALF-LENGTH OF THE INTERVAL (BL,BR)
C           CENTR  -MID POINT OF THE INTERVAL (BL,BR)
C
C***FIRST EXECUTABLE STATEMENT
      NEV = 25
      IF(BL.EQ.A.AND.(ALFA.NE.0.0D+00.OR.INTEGR.EQ.2.OR.INTEGR.EQ.4))
     * GO TO 10
      IF(BR.EQ.B.AND.(BETA.NE.0.0D+00.OR.INTEGR.EQ.3.OR.INTEGR.EQ.4))
     * GO TO 140
C
C           IF A.GT.BL AND B.LT.BR, APPLY THE 15-POINT GAUSS-KRONROD
C           SCHEME.
C
C
      CALL QK15W(F,QWGTS,A,B,ALFA,BETA,INTEGR,BL,BR,RESULT,ABSERR,
     *  RESABS,RESASC)
      NEV = 15
      GO TO 270
C
C           THIS PART OF PROGRAM IS EXECUTED ONLY IF A=BL.
C           ---------------------------------------------
C
C           COMPUTE THE CHEBYSHEV SERIES EXPANSION OF THE
C           FOLLOWING FUNCTION
C           F1 = (0.5*(B+B-BR-A)-0.5*(BR-A)*X)**BETA
C                 *F(0.5*(BR-A)*X+0.5*(BR+A))
C
   10 HLGTH = 5.0D-01*(BR-BL)
      CENTR = 5.0D-01*(BR+BL)
      FIX = B-CENTR
      FVAL(1) = 5.0D-01*F(HLGTH+CENTR)*(FIX-HLGTH)**BETA
      FVAL(13) = F(CENTR)*(FIX**BETA)
      FVAL(25) = 5.0D-01*F(CENTR-HLGTH)*(FIX+HLGTH)**BETA
      DO 20 I=2,12
        U = HLGTH*X(I-1)
        ISYM = 26-I
        FVAL(I) = F(U+CENTR)*(FIX-U)**BETA
        FVAL(ISYM) = F(CENTR-U)*(FIX+U)**BETA
   20 CONTINUE
      FACTOR = HLGTH**(ALFA+1.0D+00)
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      RES12 = 0.0D+00
      RES24 = 0.0D+00
      IF(INTEGR.GT.2) GO TO 70
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
C
C           INTEGR = 1  (OR 2)
C
      DO 30 I=1,13
        RES12 = RES12+CHEB12(I)*RI(I)
        RES24 = RES24+CHEB24(I)*RI(I)
   30 CONTINUE
      DO 40 I=14,25
        RES24 = RES24+CHEB24(I)*RI(I)
   40 CONTINUE
      IF(INTEGR.EQ.1) GO TO 130
C
C           INTEGR = 2
C
      DC = DLOG(BR-BL)
      RESULT = RES24*DC
      ABSERR = DABS((RES24-RES12)*DC)
      RES12 = 0.0D+00
      RES24 = 0.0D+00
      DO 50 I=1,13
        RES12 = RES12+CHEB12(I)*RG(I)
        RES24 = RES12+CHEB24(I)*RG(I)
   50 CONTINUE
      DO 60 I=14,25
        RES24 = RES24+CHEB24(I)*RG(I)
   60 CONTINUE
      GO TO 130
C
C           COMPUTE THE CHEBYSHEV SERIES EXPANSION OF THE
C           FOLLOWING FUNCTION
C           F4 = F1*DLOG(0.5*(B+B-BR-A)-0.5*(BR-A)*X)
C
   70 FVAL(1) = FVAL(1)*ALOG(FIX-HLGTH)
      FVAL(13) = FVAL(13)*DLOG(FIX)
      FVAL(25) = FVAL(25)*DLOG(FIX+HLGTH)
      DO 80 I=2,12
        U = HLGTH*X(I-1)
        ISYM = 26-I
        FVAL(I) = FVAL(I)*DLOG(FIX-U)
        FVAL(ISYM) = FVAL(ISYM)*DLOG(FIX+U)
   80 CONTINUE
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
C
C           INTEGR = 3  (OR 4)
C
      DO 90 I=1,13
        RES12 = RES12+CHEB12(I)*RI(I)
        RES24 = RES24+CHEB24(I)*RI(I)
   90 CONTINUE
      DO 100 I=14,25
        RES24 = RES24+CHEB24(I)*RI(I)
  100 CONTINUE
      IF(INTEGR.EQ.3) GO TO 130
C
C           INTEGR = 4
C
      DC = DLOG(BR-BL)
      RESULT = RES24*DC
      ABSERR = DABS((RES24-RES12)*DC)
      RES12 = 0.0D+00
      RES24 = 0.0D+00
      DO 110 I=1,13
        RES12 = RES12+CHEB12(I)*RG(I)
        RES24 = RES24+CHEB24(I)*RG(I)
  110 CONTINUE
      DO 120 I=14,25
        RES24 = RES24+CHEB24(I)*RG(I)
  120 CONTINUE
  130 RESULT = (RESULT+RES24)*FACTOR
      ABSERR = (ABSERR+DABS(RES24-RES12))*FACTOR
      GO TO 270
C
C           THIS PART OF THE PROGRAM IS EXECUTED ONLY IF B=BR.
C           --------------------------------------------------
C
C           COMPUTE THE CHEBYSHEV SERIES  EXPANSION OF THE
C           FOLLOWING FUNCTION
C           F2 = (0.5*(B+BL-A-A)+0.5*(B-BL)*X)**ALFA
C                *F(0.5*(B-BL)*X+0.5*(B+BL))
C
  140 HLGTH = 5.0D-01*(BR-BL)
      CENTR = 5.0D-01*(BR+BL)
      FIX = CENTR-A
      FVAL(1) = 5.0D-01*F(HLGTH+CENTR)*(FIX+HLGTH)**ALFA
      FVAL(13) = F(CENTR)*(FIX**ALFA)
      FVAL(25) = 5.0D-01*F(CENTR-HLGTH)*(FIX-HLGTH)**ALFA
      DO 150 I=2,12
        U = HLGTH*X(I-1)
        ISYM = 26-I
        FVAL(I) = F(U+CENTR)*(FIX+U)**ALFA
        FVAL(ISYM) = F(CENTR-U)*(FIX-U)**ALFA
  150 CONTINUE
      FACTOR = HLGTH**(BETA+1.0D+00)
      RESULT = 0.0D+00
      ABSERR = 0.0D+00
      RES12 = 0.0D+00
      RES24 = 0.0D+00
      IF(INTEGR.EQ.2.OR.INTEGR.EQ.4) GO TO 200
C
C           INTEGR = 1   (OR 3)
C
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
      DO 160 I=1,13
        RES12 = RES12+CHEB12(I)*RJ(I)
        RES24 = RES24+CHEB24(I)*RJ(I)
  160 CONTINUE
      DO 170 I=14,25
        RES24 = RES24+CHEB24(I)*RJ(I)
  170 CONTINUE
      IF(INTEGR.EQ.1) GO TO 260
C
C           INTEGR = 3
C
      DC = DLOG(BR-BL)
      RESULT = RES24*DC
      ABSERR = DABS((RES24-RES12)*DC)
      RES12 = 0.0D+00
      RES24 = 0.0D+00
      DO 180 I=1,13
        RES12 = RES12+CHEB12(I)*RH(I)
        RES24 = RES24+CHEB24(I)*RH(I)
  180 CONTINUE
      DO 190 I=14,25
        RES24 = RES24+CHEB24(I)*RH(I)
  190 CONTINUE
      GO TO 260
C
C           COMPUTE THE CHEBYSHEV SERIES EXPANSION OF THE
C           FOLLOWING FUNCTION
C           F3 = F2*DLOG(0.5*(B-BL)*X+0.5*(B+BL-A-A))
C
  200 FVAL(1) = FVAL(1)*DLOG(HLGTH+FIX)
      FVAL(13) = FVAL(13)*DLOG(FIX)
      FVAL(25) = FVAL(25)*DLOG(FIX-HLGTH)
      DO 210 I=2,12
        U = HLGTH*X(I-1)
        ISYM = 26-I
        FVAL(I) = FVAL(I)*DLOG(U+FIX)
        FVAL(ISYM) = FVAL(ISYM)*DLOG(FIX-U)
  210 CONTINUE
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
C
C           INTEGR = 2   (OR 4)
C
      DO 220 I=1,13
        RES12 = RES12+CHEB12(I)*RJ(I)
        RES24 = RES24+CHEB24(I)*RJ(I)
  220 CONTINUE
      DO 230 I=14,25
        RES24 = RES24+CHEB24(I)*RJ(I)
  230 CONTINUE
      IF(INTEGR.EQ.2) GO TO 260
      DC = DLOG(BR-BL)
      RESULT = RES24*DC
      ABSERR = DABS((RES24-RES12)*DC)
      RES12 = 0.0D+00
      RES24 = 0.0D+00
C
C           INTEGR = 4
C
      DO 240 I=1,13
        RES12 = RES12+CHEB12(I)*RH(I)
        RES24 = RES24+CHEB24(I)*RH(I)
  240 CONTINUE
      DO 250 I=14,25
        RES24 = RES24+CHEB24(I)*RH(I)
  250 CONTINUE
  260 RESULT = (RESULT+RES24)*FACTOR
      ABSERR = (ABSERR+DABS(RES24-RES12))*FACTOR
  270 RETURN
      END



