      SUBROUTINE QK15(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C......................................................................
C
C 1.     QK15
C        INTEGRATION RULES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE  I = INTEGRAL OF F OVER (A,B),WITH ERROR
C                           ESTIMATE
C                       J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.     CALLING SEQUENCE
C           CALL QK15(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C           PARAMETERS
C            ON ENTRY
C              F      -REAL
C                      FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                      FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                      DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C              A      -REAL
C                      LOWER LIMIT OF INTEGRATION
C
C              B      -REAL
C                      UPPER LIMIT OF INTEGRATION
C
C            ON RETURN
C              RESULT -REAL
C                      APPROXIMATION TO THE INTEGRAL I
C                      RESULT IS COMPUTED BY APPLYING THE 15-POINT
C                      KRONROD RULE (RESK) OBTAINED BY OPTIMAL ADDITION
C                      OF ABSCISSAE TO THE 7-POINT GAUSS RULE (RESG).
C
C              ABSERR -REAL
C                      ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                      WHICH SHOULD NOT EXCEED ABS(I-RESULT)
C
C              RESABS -REAL
C                      APPROXIMATION TO THE INTEGRAL J
C
C              RESASC -REAL
C                      APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C                      OVER (A,B)
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C                 - F (USER-PROVIDED FUNCTION)
C                 - QMACO
C                 - FORTRAN ABS, AMAX1,AMIN1
C
C.....................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(7),FV2(7),WG(4),WGK(8),XGK(8)
C
C           THE ABSCISSAE AND WEIGHTS ARE GIVEN FOR THE INTERVAL (-1,1)
C           BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C           CORRESPONDING WEIGHTS ARE GIVEN.
C
C           XGK   - ABSCISAE OF THE 15-POINT KRONROD RULE
C                   XGK(2), XGK(4), ... ABSCISSAE OF THE 7-POINT
C                   GAUSS RULE
C                   XGK(1), XGK(3), ... ABSCISSAE WHICH ARE OPTIMALLY
C                   ADDED TO THE 7-POINT GAUSS RULE
C
C           WGK   - WEIGHTS OF THE 15-POINT KRONROD RULE
C
C           WG    - WEIGHTS OF THE 7-POINT GAUSS RULE
C
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8)/
     *     9.914553711208126D-01,      9.491079123427585D-01,
     *     8.648644233597691D-01,      7.415311855993944D-01,
     *     5.860872354676911D-01,      4.058451513773972D-01,
     *     2.077849550078985D-01,      0.0E+00              /
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8)/
     *     2.293532201052922D-02,      6.309209262997855D-02,
     *     1.047900103222502D-01,      1.406532597155259D-01,
     *     1.690047266392697D-01,      1.903505780647854D-01,
     *     2.044329400752989D-01,      2.094821410847278D-01/
      DATA WG(1),WG(2),WG(3),WG(4)/
     *     1.294849661688697D-01,      2.797053914892767D-01,
     *     3.818300505051189D-01,      4.179591836734694D-01/
C
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           CENTR  - MID POINT OF THE INTERVAL
C           HLGTH  - HALF-LENGTH OF THE INTERVAL
C           ABSC   - ABSCISSA
C           FVAL*  - FUNCTION VALUE
C           RESG   - RESULT OF THE 7-POINT GAUSS FORMULA
C           RESK   - RESULT OF THE 15-POINT KRONROD FORMULA
C           RESKH  - APPROXIMATION TO THE MEAN VALUE OF F OVER (A,B),
C                    I.E. TO I/(B-A)
C
C           MACHINE DEPENDENT COSTANTS
C           --------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(A+B)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C           COMPUTE THE 15-POINT KRONROD APPROXIMATION TO THE INTEGRAL,
C           AND ESTIMATE THE ABSOLUTE ERROR.
C
      FC = F(CENTR)
      RESG = FC*WG(4)
      RESK = FC*WGK(8)
      RESABS = DABS(RESK)
      DO 10 J=1,3
        JTW = J*2
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG = RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15 J=1,4
        JTWM1 = J*2-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(DABS(FVAL1)+DABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0E-01
      RESASC = WGK(8)*DABS(FC-RESKH)
      DO 20 J=1,7
        RESASC = RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END
