      SUBROUTINE QK15I(F,BOUN,INF,A,B,RESULT,ABSERR,RESABS,RESASC)
C......................................................................
C
C  1.      QK15I
C          INTEGRATION RULE
C              STANDARD FORTRAN SUBROUTINE
C
C  2.      PURPOSE
C              THE ORIGINAL (INFINITE) INTEGATION RANGE IS MAPPED
C              ONTO THE INTERVAL (0,1) AND (A,B) IS A PART OF (0,1).
C              IT IS THE PURPOSE TO COMPUTE
C              I=INTEGRAL OF TRANSFORMED INTEGRAND OVER (A,B),
C              J=INTEGRAL OF ABS(TRANSFORMED INTEGRAND) OVER (A,B).
C
C  3.      CALLING SEQUENCE
C              CALL QK15I(F,BOUN,INF,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C          PARAMETERS
C            ON ENTRY
C              F       -REAL
C                       FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                       FUNCTION F(X).THE ACTUAL NAME FOR F NEEDS TO BE
C                       DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C              BOUN    -REAL
C                       FINITE BOUND OF ORIGINAL INTEGRATION RANGE
C                       (SET TO ZERO IF INF = +2)
C
C              INF     -INTEGER
C                       IF INF = -1, THE ORIGINAL INTERVAL IS
C                                   (-INFINITY,BOUND),
C                       IF INF = +1, THE ORIGINAL INTERVAL IS
C                                   (BOUND,+INFINITY),
C                       IF INF = +2, THE ORIGINAL INTERVAL IS
C                                   (-INFINITY,+INFINITY)
C                       THE INTEGRAL IS COMPUTED AS THE SUM OF TWO
C                       INTEGRALS, ONE OVER (-INFINITY,0) AND ONE
C                       OVER (0,+INFINITY).
C
C              A       -REAL
C                       LOWER LIMIT FOR INTEGRATION OVER SUBRANGE
C                       OF (0,1)
C
C              B       -REAL
C                       UPPER LIMIT FOR INTEGRATION OVER SUBRANGE
C                       OF (0,1)
C            ON RETURN
C              RESULT  -REAL
C                       APPROXIMATION TO THE INTEGRAL I
C                       RESULT IS COMPUTED BY APPLYNG THE 15-POINT
C                       KRONROD RULE(RESK) OBTAINED BY OPTIMAL ADDITION
C                       OF ABSCISSAE TO THE 7-POINT GAUSS RULE (RESG).
C
C              ABSERR  -REAL
C                       ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                       WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C              RESABS  -REAL
C                       APPROXIMATION TO THE INTEGRAL J
C
C              RESASC  -REAL
C                       APPROXIMATION TO THE INTEGRAL OF
C                       ABS((TRANSFORMED INTEGRAND)-I/(B-A)) OVER (A,B)
C
C 4.       SUBROUTINES OR FUNCTIONS NEEDED
C                  -F (USER-PROVIDED FUNCTION)
C                  -QMACO
C                  -FORTRAN ABS, AMAX1, AMIN1, MIN0
C
C......................................................................
C
      REAL*8 A,ABSC,ABSC1,ABSC2,ABSERR,B,BOUN,CENTR,DINF,EPMACH,F,FC,
     *  FSUM,FVAL1,FVAL2,FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,
     *  RESKH,RESULT,TABSC1,TABSC2,UFLOW,WG,WGK,XGK
      INTEGER INF,J
C
      DIMENSION FV1(7),FV2(7),WGK(8),WG(8),XGK(8)
C
C          THE ABSCISSAE AND WEIGHTS ARE SUPPLIED FOR THE INTERVAL
C          (-1,1). BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND
C          THEIR CORRESPONDING WEIGHTS ARE GIVEN.
C
C          XGK     -ABSCISSAE OF THE 15-POINT KRONROD RULE
C                   XGK(2), XGK(4), ...ABSCISSAE OF THE 7-POINT GAUSS
C                   RULE
C                   XGK(1), XGK(3), ...ABSCISSAE WHICH ARE OPTIMALLY
C                   ADDED TO THE 7-POINT GAUSS RULE
C
C          WGK     -WEIGHTS OF THE 15-POINT KRONROD RULE
C
C          WG      -WEIGHTS OF THE 7-POINT GAUSS RULE, CORRESPONDING
C                   TO THE ABSCISSAE XGK(2), XGK(4), ...
C                   WG(1), WG(3), ... ARE SET TO ZERO.
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8)/
     *     9.914553711208126D-01,     9.491079123427585D-01,
     *     8.648644233597691D-01,     7.415311855993944D-01,
     *     5.860872354676911D-01,     4.058451513773972D-01,
     *     2.077849550078985D-01,     0.000000000000000D+00/
C
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8)/
     *     2.293532201052922D-02,     6.309209262997855D-02,
     *     1.047900103222502D-01,     1.406532597155259D-01,
     *     1.690047266392679D-01,     1.903505780647854D-01,
     *     2.044329400752989D-01,     2.094821410847278D-01/
C
      DATA WG(1),WG(2),WG(3),WG(4),WG(5),WG(6),WG(7),WG(8)/
     *     0.0000000000000000D+00,    1.294849661688697D-01,
     *     0.0000000000000000D+00,    2.797053914892767D-01,
     *     0.0000000000000000D+00,    3.818300505051189D-01,
     *     0.0000000000000000D+00,    4.179591836734694D-01/
C
C
C             LIST OF MAJOR VARIABLES
C             -----------------------
C
C             CENTR   -MID POINT OF THE INTERVAL
C             HLGTH   -HALF-LENGTH OF THE INTERVAL
C             ABSC*   -ABSCISSA
C             TABSC*  -TRANSFORMED ABSCISSA
C             FVAL*   -FUNCTION VALUE
C             RESG    -RESULT OF THE 7-POINT GAUSS FORMULA
C             RESK    -RESULT OF THE 15-POINT KRONROD FORMULA
C             RESKH   -APPROXIMATION TO THE MEAN VALUE OF THE TRANSFORMED
C                      INTEGRAND OVER (A,B), I.E. TO I/(B-A)
C
C             MACHINE DEPENDENT CONSTANTS
C             ---------------------------
C
C             EPMACH IS THE LARGEST RELATIVE SPACING.
C             UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C             OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRSTEXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
      DINF =MIN0(1,INF)
C
      CENTR =5.0D-01*(A+B)
      HLGTH =5.0D-01*(B-A)
      TABSC1 =BOUN+DINF*(1.0D+00-CENTR)/CENTR
      FVAL1 =F(TABSC1)
      IF(INF.EQ.2) FVAL1 =FVAL1+F(-TABSC1)
      FC =(FVAL1/CENTR)/CENTR
C
C            COMPUTE THE 15-POINT KRONROD APPROXIMATION TO THE INTEGRAL,
C            AND ESTIMATE THE ERROR.
C
      RESG =WG(8)*FC
      RESK =WGK(8)*FC
      RESABS =DABS(RESK)
      DO 10 J=1,7
        ABSC =HLGTH*XGK(J)
        ABSC1 =CENTR-ABSC
        ABSC2 =CENTR+ABSC
        TABSC1 =BOUN+DINF*(1.0D+00-ABSC1)/ABSC1
        TABSC2 =BOUN+DINF*(1.0D+00-ABSC2)/ABSC2
        FVAL1 =F(TABSC1)
        FVAL2 =F(TABSC2)
        IF(INF.EQ.2) FVAL1=FVAL1+F(-TABSC1)
        IF(INF.EQ.2) FVAL2=FVAL2+F(-TABSC2)
        FVAL1 =(FVAL1/ABSC1)/ABSC1
        FVAL2 =(FVAL2/ABSC2)/ABSC2
        FV1(J) =FVAL1
        FV2(J) =FVAL2
        FSUM =FVAL1+FVAL2
        RESG =RESG+WG(J)*FSUM
        RESK =RESK+WGK(J)*FSUM
        RESABS =RESABS+WGK(J)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      RESKH =RESK*5.0D-01
      RESASC =WGK(8)*DABS(FC-RESKH)
      DO 20 J=1,7
        RESASC =RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT =RESK*HLGTH
      RESASC =RESASC*HLGTH
      RESABS =RESABS*HLGTH
      ABSERR =DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00) ABSERR =RESASC*
     *  DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR =DMAX1
     * ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END
