$DEBUG
      SUBROUTINE QK21(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C......................................................................
C
C 1.     QK21
C        INTEGRATION RULES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE  I = INTEGRAL OF F OVER (A,B), WITH ERROR
C                           ESTIMATE
C                       J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.     CALLING SEQUENCE
C            CALL QK21(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C        PARAMETERS
C         ON ENTRY
C           F     -REAL
C                  FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                  FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                  DECLARED E X T E R N A L  IN THE CALLING PROGRAM.
C
C           A     -REAL
C                  LOWER LIMIT OF INEGRATION
C
C           B     -REAL
C                  UPPER LIMIT OF INTEGRATION
C
C         ON RETURN
C           RESULT-REAL
C                  APPROXIMATION TO THE INTEGRAL I
C                  RESULT IS COMPUTED BY APPLYING THE 21-POINT
C                  KRONROD RULE (RESK) OBTAINED BY OPTIMAL ADDITION
C                  OF ABSCISSAE TO THE 10-POINT GAUSS RULE (RESG).
C
C           ABSERR-REAL
C                  ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                  WHICH SHOULD NOT EXCEED ABS(I-RESULT)
C
C           RESABS-REAL
C                  APPROXIMATION TO THE INTEGRAL J
C
C           RESASC-REAL
C                  APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C                  OVER (A,B)
C
C 4.     SUBROUTINES OR FUNCTION NEEDED
C               - F (USER-PROVIDED FUNCTION)
C               - QMACO
C               - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(10),FV2(10),WG(5),WGK(11),XGK(11)
C
C           THE ABSISSAE AND WEIGHTS ARE GIVEN FOR THE INTERVAL (-1,1)
C           BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C           CORRESPONDING WEIGHTS ARE GIVEN.
C
C           XGK    -ABSCISSAE OF THE 21-POINT KRONROD RULE
C                   XGK(2), XGK(4), ... ABSCISSAE OF THE 10-POINT
C                   GAUSS RULE
C                   XGK(1), XGK(3), ... ABSCISSAE WHICH ARE OPTIMALLY
C                   ADDED TO THE 10-POINT GAUSS RULE
C
C           WGK    -WEIGHTS OF THE 21-POINT KRONROD RULE
C
C           WG     -WEIGHTS OF THE 10-POINT GAUSS RULE
C
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8),
     *     XGK(9),XGK(10),XGK(11)/
     *     9.956571630258081D-01,      9.739065285171717D-01,
     *     9.301574913557082D-01,      8.650633666889845D-01,
     *     7.808177265864169D-01,      6.794095682990244D-01,
     *     5.627571346686047D-01,      4.333953941292472D-01,
     *     2.943928627014602D-01,      1.488743389816312D-01,
     *     0.000000000000000D+00/
C
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8),
     *     WGK(9),WGK(10),WGK(11)/
     *     1.169463886737187D-02,      3.255816230796473D-02,
     *     5.475589657435200D-02,      7.503967481091995D-02,
     *     9.312545458369761D-02,      1.093871588022976D-01,
     *     1.234919762620659D-01,      1.347092173114733D-01,
     *     1.427759385770601D-01,      1.477391049013385D-01,
     *     1.494455540029169D-01/
C
      DATA WG(1),WG(2),WG(3),WG(4),WG(5)/
     *     6.667134430868814D-02,      1.494513491505806D-01,
     *     2.190863625159820D-01,      2.692667193099964D-01,
     *     2.955242247147529D-01/
C
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           CENTR  - MID POINT OF THE INTERVAL
C           HLGTH  - HALF-LENGTH OF THE INTERVAL
C           ABSC   - ABSCISSA
C           FVAL*  - FUNCTION VALUE
C           RESG   - RESULT OF THE 10-POINT GAUSS FORMULA
C           RESK   - RESULT OF THE 21-POINT KRONROD FORMULA
C           RESKH  - APPROXIMATION TO THE MEAN VALUE OF F OVER (A,B),
C                    I.E. TO I/(B-A)
C
C
C           MACHINE DEPENDENT CONSTANTS
C           ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(A+B)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C           COMPUTE THE 21-POINT KRONROD APPROXIMATION TO THE
C           INTEGRAL, AND ESTIMATE THE ABSOLUTE ERROR.
C
      RESG = 0.0D+00
      FC = F(CENTR)
      RESK = WGK(11)*FC
      RESABS = DABS(RESK)
      DO 10 J=1,5
        JTW = 2*J
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG = RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15 J=1,5
        JTWM1 = 2*J-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(DABS(FVAL1)+DABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0D-01
      RESASC = WGK(11)*DABS(FC-RESKH)
      DO 20 J=1,10
        RESASC = RESASC+WGK(J)*(ABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END


