      SUBROUTINE QK31(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C.....................................................................
C
C 1.       QK31
C          INTEGRATION RULES
C             STANDARD FORTRAN SUBROUTINE
C
C 2.       PURPOSE
C             TO COMPUTE I = INTEGRAL OF F OVER (A,B), WITH ERROR
C                            ESTIMATE
C                        J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.       CALLING SEQUENCE
C             CALL QK31(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C          PARAMETERS
C           ON ENTRY
C             F       - REAL
C                       FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                       FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                       DECLARED E X T E R N A L  IN THE CALLING PROGRAM.
C
C             A       - REAL
C                       LOWER LIMIT OF INTEGRATION
C
C             B       - REAL
C                       UPPER LIMIT OF INTEGRATION
C
C          ON RETURN
C            RESULT   - REAL
C                       APPROXIMATION TO THE INTEGRAL I
C                       RESULT IS COMPUTED BY APPLYING THE 31-POINT
C                       GAUSS-KRONROD RULE (RESK), OBTAINED BY OPTIMAL
C                       ADDITION OF ABSCISSAE TO THE 15-POINT GAUSS
C                       RULE (RESG).
C
C            ABSERR   - REAL
C                       ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR
C                       WHICH SHOULD NOT EXCEED ABS(I-RESULT)
C
C            RESABS   - REAL
C                       APPROXIMATION TO THE INTERGRAL J
C
C            RESASC   - REAL
C                       APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C                       OVER (A,B)
C
C
C 4.         SUBROUTINES OR FUNCTIONS NEEDED
C                  - F (USER-PROVIDED FUNCTION)
C                  - QMACO
C                  - FORTRAN ABS, AMIN1, AMAX1
C
C................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(15),FV2(15),WG(8),WGK(16),XGK(16)
C
C           THE ABSCISSAE AND WEIGTHTS ARE GIVEN FOR THE INTERVAL (-1,1)
C           BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C           CORRESPONDING WEIGHTS ARE GIVEN.
C
C            XGK    - ABSCISSAE OF THE 31-POINT KRONROD RULE
C                     XGK(2), XGK(4), ... ABSCISSAE OF THE 15-POINT
C                     GAUSS RULE
C                     XGK(1), XGK(3), ... ABSCISSAE WHICH ARE OPTIMALLY
C                     ADDED TO THE 15-POINT GAUSS RULE
C
C            WGK    - WEIGHTS OF THE 31-POINT KRONROD RULE
C
C            WG     - WEIGHTS OF THE 15-POINT GAUSS RULE
C
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8),
     *  XGK(9),XGK(10),XGK(11),XGK(12),XGK(13),XGK(14),XGK(15),XGK(16)/
     *     9.980022986933971D-01,   9.879925180204854D-01,
     *     9.677390756791391D-01,   9.372733924007059D-01,
     *     8.972645323440819D-01,   8.482065834104272D-01,
     *     7.904185014424659D-01,   7.244177313601700D-01,
     *     6.509967412974170D-01,   5.709721726085388D-01,
     *     4.850818636402397D-01,   3.941513470775634D-01,
     *     2.991800071631688D-01,   2.011940939974345D-01,
     *     1.011420669187175D-01,   0.0E+00               /
C
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8),
     *  WGK(9),WGK(10),WGK(11),WGK(12),WGK(13),WGK(14),WGK(15),WGK(16)/
     *     5.377479872923349D-03,   1.500794732931612D-02,
     *     2.546084732671532D-02,   3.534636079137585D-02,
     *     4.458975132476488D-02,   5.348152469092809D-02,
     *     6.200956780067064D-02,   6.985412131872826D-02,
     *     7.684968075772038D-02,   8.308050282313302D-02,
     *     8.856444305621177D-02,   9.312659817082532D-02,
     *     9.664272698362368D-02,   9.917359872179196D-02,
     *     1.007698455238756D-01,   1.013300070147915D-01/
C
      DATA WG(1),WG(2),WG(3),WG(4),WG(5),WG(6),WG(7),WG(8)/
     *     3.075324199611727D-02,   7.036604748810812D-02,
     *     1.071592204671719D-01,   1.395706779261543D-01,
     *     1.662692058169939D-01,   1.861610000155622D-01,
     *     1.984314853271116D-01,   2.025782419255613D-01/
C
C
C          LIST OF MAJOR VARIABLES
C          -----------------------
C          CENTR  - MID POINT OF THE INTERVAL
C          HLGTH  - HALF-LENGTH OF THE INTERVAL
C          ABSC   - ABSCISSA
C          FVAL*  - FUNCTION VALUE
C          RESG   - RESULT OF THE 15-POINT GAUSS FORMULA
C          RESK   - RESULT OF THE 31-POINT KRONROD FORMULA
C          RESKH  - APPROXIMATION TO THE MEAN VALUE OF F OVER (A,B),
C                   I.E. TO I/(B-A)
C
C          MACHINE DEPENDENT CONSTANTS
C          ---------------------------
C          EPMACH IS THE LARGEST RELATIVE SPACING.
C          UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C          OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(A+B)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C          COMPUTE THE 31-POINT KRONROD APPROXIMATION TO THE INTEGRAL,
C          AND ESTIMATE THE ABSOLUTE ERROR.
C
      FC = F(CENTR)
      RESG = WG(8)*FC
      RESK = WGK(16)*FC
      RESABS = DABS(RESK)
      DO 10  J = 1,7
        JTW = J*2
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG = RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15  J = 1,8
        JTWM1 = J*2-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(DABS(FVAL1)+DABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0D-01
      RESASC = WGK(16)*DABS(FC-RESKH)
      DO 20  J = 1,15
        RESASC = RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END

