      SUBROUTINE QK41(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C......................................................................
C
C 1.     QK41
C        INTEGRATION RULES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE I = INTEGRAL OF F OVER (A,B), WITH ERROR
C                          ESTIMATE
C                      J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.     CALLING SEQUENCE
C           CALL QK41(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C        PARAMETERS
C         ON ENTRY
C           F        - REAL
C                      FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                      FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                      DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C           A        - REAL
C                      LOWER LIMIT OF INTEGRATION
C
C           B        - REAL
C                      UPPER LIMIT OF ITEGRATION
C
C        ON RETURN
C          RESULT    - REAL
C                      APPROXIMATION TO THE INTEGRAL I
C                      RESULT IS COMPUTED BY APPLYING THE 41-POINT
C                      GAUSS-KRONROD RULE (RESK) OBTAINED BY OPTIMAL
C                      ADDITION OF ABSCISSAE TO THE 20-POINT GAUSS
C                      RULE (RESG).
C
C          ABSERR    - REAL
C                      ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                      WHICH SHOULD NOT EXCEED ABS(I-RESULT)
C
C          RESABS    - REAL
C                      APPROXIMATION TO THE INTEGRAL J
C
C          RESASC    - REAL
C                      APPROXIMATION TO THE INTEGRAL OF ABS (F-I/(B-A))
C                      OVER (A,B)
C
C 4.       SUBROUTINES OR FUNCTIONS NEEDED
C                    - F (USER-PROVIDED FUNCTION)
C                    - QMACO
C                    - FORTRAN ABS, AMAX1, AMIN1
C
C......................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(20),FV2(20),WG(10),WGK(21),XGK(21)
C
C          THE ABSCISSAE AND WEIGHTS ARE GIVEN FOR THE INTERVAL (-1,1)
C          BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C          CORRESPONDING WEIGHTS ARE GIVEN.
C
C          XGK    - ABSCISSAE OF THE 41-POINT GAUSS-KRONROD RULE
C                   XGK(2), XGK(4), ... ABSCISSAE OF THE 20-POINT
C                   GAUSS RULE
C                   XGK(1), XGK(3), ... ABSCISSAE WHICH ARE OPTIMALLY
C                   ADDED TO THE 20-POINT GAUSS RULE
C
C          WGK    - WEIGHTS OF THE 41-POINT GAUSS-KRONROD RULE
C
C          WG     - WEIGHTS OF THE 20-POINT GAUSS RULE
C
      DATA XGK  /
     *     9.988590315882777D-01,   9.931285991850949D-01,
     *     9.815078774502503D-01,   9.639719272779138D-01,
     *     9.408226338317548D-01,   9.122344282513259D-01,
     *     8.782768112522820D-01,   8.391169718222188D-01,
     *     7.950414288375512D-01,   7.463319064601508D-01,
     *     6.932376563347514D-01,   6.360536807265150D-01,
     *     5.751404468197103D-01,   5.108670019508271D-01,
     *     4.435931752387251D-01,   3.737060887154196D-01,
     *     3.016278681149130D-01,   2.277858511416451D-01,
     *     1.526054652409227D-01,   7.652652113349733D-02,
     *     0.0E+00              /
C
      DATA WGK  /
     *     3.073583718520532D-03,    8.600269855642942D-03,
     *     1.462616925697125D-02,    2.038837346126652D-02,
     *     2.588213360495116D-02,    3.128730677703280D-02,
     *     3.660016975820080D-02,    4.166887332797369D-02,
     *     4.643482186749767D-02,    5.094457392372869D-02,
     *     5.519510534828599D-02,    5.911140088063957D-02,
     *     6.265323755478117D-02,    6.583459713361842D-02,
     *     6.864867292852162D-02,    7.105442355344407D-02,
     *     7.303069033278667D-02,    7.458287540049919D-02,
     *     7.570449768455667D-02,    7.637786767208074D-02,
     *     7.660071191799966D-02/
C
      DATA WG  /
     *     1.761400713915212D-02,    4.060142980038694D-02,
     *     6.267204833410906D-02,    8.327674157670475D-02,
     *     1.019301198172404D-01,    1.181945319615184D-01,
     *     1.316886384491766D-01,    1.420961093183821D-01,
     *     1.491729864726037D-01,    1.527533871307259D-01/
C
C
C          LIST OF MAJOR VARIABLES
C          -----------------------
C
C          CENTR   -MID POINT OF THE INTERVAL
C          HLGTH   -HALF-LENGTH OF THE INTERVAL
C          ABSC    -ABSCISSA
C          FVAL*   -FUNCTION VALUE
C          RESG    -RESULT OF THE 20-POINT GAUSS FORMULA
C          RESK    -RESULT OF THE 41-POINT KRONROD FORMULA
C          RESKH   -APPROXIMATION TO MEAN VALUE OF F OVER (A,B), I.E.
C                   TO I/(B-A)
C
C          MACHINE DEPENDENT CONSTANTS
C          ---------------------------
C
C          EPMACH IS THE LARGEST RELATIVE SPACING.
C          UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C          OFLOW IS THE LARGEST MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(A+B)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C          COMPUTE 41-POINT GAUSS-KRONROD APPROXIMATION TO THE
C          INTEGRAL,AND ESTIMATE THE ABSOLUTE ERROR.
C
      RESG = 0.0D+00
      FC = F(CENTR)
      RESK = WGK(21)*FC
      RESABS = DABS(RESK)
      DO 10 J=1,10
        JTW =J*2
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG = RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15 J=1,10
        JTWM1 = J*2-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(ABS(FVAL1)+ABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0D-01
      RESASC = WGK(21)*DABS(FC-RESKH)
      DO 20 J=1,20
        RESASC = RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END

