      SUBROUTINE QK51(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C.....................................................................
C
C 1.     QK51
C        INTEGRATION RULES
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE  I = INTEGRAL OF F OVER (A,B), WITH ERROR
C                           ESTIMATE
C                       J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.     CALLING SEQUENCE
C           CALL QK51(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C        PARAMETERS
C         ON ENTRY
C           F     - REAL
C                   FUNCTION SUBROUTINE DEFINING THE INTEGRAND
C                   FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                   DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C           A     - REAL
C                   LOWER LIMIT OF INTERATION
C
C           B     - REAL
C                   UPPER LIMIT OF INTEGRATION
C
C         ON RETURN
C           RESULT- REAL
C                   APPROXIMATION TO THE INTEGRAL I
C                   RESULT IS COMPUTED BY APPLYING THE 51-POINT
C                   KRONROD RULE (RESK) OBTAINED BY OPTIMAL ADDITION
C                   OF ABSCISSAE TO THE 25-POINT GAUSS RULE (RESG).
C
C           ABSERR- REAL
C                   ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                   WHICH SHOULD NOT EXCEED ABS(I-RESULT)
C
C           RESABS -REAL
C                   APPROXIMATION TO THE INTEGRAL J
C
C           RESASC -REAL
C                   APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C                   OVER (A,B)
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C
C               - F (USER-PROVIDED FUNCTION)
C               - QMACO
C               - FORTRAN ABS, AMAX1, AMIN1
C
C................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
C
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(25),FV2(25),XGK(26),WGK(26),WG(13)
C
C          THE ABSCISSAE AND WEIGHTS ARE GIVEN FOR THE INTERVAL (-1,1).
C          BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C          CORRESPONDING WEIGHTS ARE GIVEN.
C
C          XGK    - ABSCISSAE OF THE 51-POINT KRONROD RULE
C                   XGK(2), XGK(4), ... ABSCISSAE OF THE 25-POINT
C                   GAUSS RULE
C                   XGK(1), XGK(3), ... ABSCISSAE WHICH ARE OPTIMALLY
C                   ADDED TO THE 25-POINT GAUSS RULE
C
C          WGK    - WEIGHTS OF THE 51-POINT KRONROD RULE
C
C          WG     - WEIGHTS OF THE 25-POINT GAUSS RULE
C
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8),
     *  XGK(9),XGK(10),XGK(11),XGK(12),XGK(13),XGK(14)/
     *     9.992621049926098D-01,  9.955569697904981D-01,
     *     9.880357945340772D-01,  9.766639214595175D-01,
     *     9.616149864258425D-01,  9.429745712289743D-01,
     *     9.207471152817016D-01,  8.949919978782754D-01,
     *     8.658470652932756D-01,  8.334426287608340D-01,
     *     7.978737979985001D-01,  7.592592630373576D-01,
     *     7.177664068130844D-01,  6.735663684734684D-01/
C
      DATA XGK(15),XGK(16),XGK(17),XGK(18),XGK(19),XGK(20),XGK(21),
     *  XGK(22),XGK(23),XGK(24),XGK(25),XGK(26)/
     *     6.268100990103174D-01,  5.776629302412230D-01,
     *     5.263252843347192D-01,  4.730027314457150D-01,
     *     4.178853821930377D-01,  3.611723058093878D-01,
     *     3.030895389311078D-01,  2.438668837209884D-01,
     *     1.837189394210489D-01,  1.228646926107104D-01,
     *     6.154448300568508D-02,  0.0E+00 /
C
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8),
     *  WGK(9),WGK(10),WGK(11),WGK(12),WGK(13),WGK(14)/
     *     1.987383892330316D-03,  5.561932135356714D-03,
     *     9.473973386174152D-03,  1.323622919557167D-02,
     *     1.684781770912830D-02,  2.043537114588284D-02,
     *     2.400994560695322D-02,  2.747531758785174D-02,
     *     3.079230016738749D-02,  3.400213027432934D-02,
     *     3.711627148341554D-02,  4.008382550403238D-02,
     *     4.287284502017005D-02,  4.550291304992179D-02/
C
      DATA WGK(15),WGK(16),WGK(17),WGK(18),WGK(19),WGK(20),WGK(21),
     *  WGK(22),WGK(23),WGK(24),WGK(25),WGK(26)/
     *     4.798253713883671D-02,  5.027767908071567D-02,
     *     5.236288580640748D-02,  5.425112988854549D-02,
     *     5.595081122041232D-02,  5.743711636156783D-02,
     *     5.868968002239421D-02,  5.972034032417406D-02,
     *     6.053945537604586D-02,  6.112850971705305D-02,
     *     6.147118987142532D-02,  6.158081806783294D-02/
C
      DATA WG(1),WG(2),WG(3),WG(4),WG(5),WG(6),WG(7),WG(8),WG(9),
     *  WG(10),WG(11),WG(12),WG(13)/
     *     1.139379850102629D-02,  2.635498661503214D-02,
     *     4.093915670130631D-02,  5.490469597583519D-02,
     *     6.803833381235692D-02,  8.014070033500102D-02,
     *     9.102826198296365D-02,  1.005359490670506D-01,
     *     1.085196244742637D-01,  1.148582591457116D-01,
     *     1.194557635357848D-01,  1.222424429903100D-01,
     *     1.231760537267155D-01/
C
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C
C           CENTR   - MID POINT OF THE INTERVAL
C           HLGTH   - HALF-LENGTH OF THE INTERVAL
C           ABSC    - ABSCISSA
C           FVAL*   - FUNCTION VALUE
C           RESG    - RESULT OF THE 25-POINT GAUSS FORMULA
C           RESK    - RESULT OF THE 51-POINT KRONROD FORMULA
C           RESKH   - APPROXIMATION TO THE MEAN VALUE OF F OVER (A,B),
C                     I.E. TO I/(B-A).
C
C           MACHINE DEPENDENT CONSTANTS
C           ---------------------------
C
C           EPMACH IS THE LARGEST RELATIVE SPACING.
C           UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C           OFLOW IS THE LARGEST POSITIVE MAGNITUDE.
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(A+B)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C         COMPUTE THE 51-POINT KRONROD APPROXIMATION TO THE INTEGRAL,
C         AND ESTIMATE THE ABSOLUTE ERROR.
C
      FC = F(CENTR)
      RESG = WG(13)*FC
      RESK = WGK(26)*FC
      RESABS = DABS(RESK)
      DO 10 J=1,12
        JTW = J*2
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG =RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15 J=1,13
        JTWM1 = J*2-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(DABS(FVAL1)+DABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0D-01
      RESASC = WGK(26)*DABS(FC-RESKH)
      DO 20 J=1,25
        RESASC = RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END


