      SUBROUTINE QK61(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C......................................................................
C
C 1.    QK61
C       INTEGRATION RULE
C          STANDARD FORTRAN SUBROUTINE
C
C 2.    PURPOSE
C          TO COMPUTE I = INTEGRAL OF F OVER (A,B), WITH ERROR
C                         ESTIMATE
C                     J = INTEGRAL OF ABS(F) OVER (A,B)
C
C 3.    CALLING SEQUENCE
C          CALL QK61(F,A,B,RESULT,ABSERR,RESABS,RESASC)
C
C       PARAMETERS
C        ON ENTRY
C          F       - REAL
C                    FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                    FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                    DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C          A       - REAL
C                    LOWER LIMIT OF INTEGRATION
C
C          B       - REAL
C                    UPPER LIMIT OF INTEGRATION
C
C        ON RETURN
C          RESULT  - REAL
C                    APPROXIMATION TO THE INTEGRAL I
C                    RESULT IS COMPUTED BY APPLYING THE 61-POINT
C                    KRONROD RULE (RESK) OBTAINED BY OPTIMAL ADDITION OF
C                    ABSCISSAE TO THE 30-POINT GAUSS RULE (RESG).
C
C          ABSERR  - REAL
C                    ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                    WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C          RESABS  - REAL
C                    APPROXIMATION TO THE INTEGRAL J
C
C          RESASC  - REAL
C                    APPROXIMATION TO THE INTEGRAL OF ABS (F-I/(B-A))
C
C
C 4.    SUBROUTINES OR FUNCTIONS NEEDED
C                  - F (USER-PROVIDED FUNCTION)
C                  - QMACO
C                  - FORTRAN ABS, AMAX1, AMIN1
C
C.....................................................................
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,F,FC,FSUM,FVAL1,FVAL2,
     *  FV1,FV2,HLGTH,OFLOW,RESABS,RESASC,RESG,RESK,RESKH,RESULT,UFLOW,
     *  WG,WGK,XGK
C
      INTEGER J,JTW,JTWM1
C
      DIMENSION FV1(30),FV2(30),XGK(31),WGK(31),WG(15)
C
C          THE ABSCISSAE AND WEIGHTS ARE GIVEN FOR THE
C          INTERVAL (-1,1). BECAUSE OF SYMMETRY ONLY THE POSITIVE
C          ABSCISSAE AND THEIR CORRESPONDING WEIGHTS ARE GIVEN.
C
C          XGK   - ABSCISSAE OF THE 61-POINT KRONROD RULE
C                  XGK(2), XGK(4) ... ABSCISSAE OF THE 3O-POINT
C                  GAUSS RULE
C                  XGK(1), XGK(3) ... OPTIMALLY ADDED ABSCISSAE
C                  TO THE 30-POINT GAUSS RULE
C
C          WGK   - WEIGHTS OF THE 61-POINT KRONROD RULE
C
C          WG    - WEIGHTS OF THE 30-POINT GAUSS RULE
C
      DATA XGK(1),XGK(2),XGK(3),XGK(4),XGK(5),XGK(6),XGK(7),XGK(8),
     *     XGK(9),XGK(10)/
     *     9.994844100504906D-01,   9.968934840746495D-01,
     *     9.916309968704046D-01,   9.836681232797472D-01,
     *     9.731163225011263D-01,   9.600218649683075D-01,
     *     9.443744447485600D-01,   9.262000474292743D-01,
     *     9.055733076999078D-01,   8.825605357920527D-01/
C
      DATA XGK(11),XGK(12),XGK(13),XGK(14),XGK(15),XGK(16),XGK(17),
     *     XGK(18),XGK(19),XGK(20)/
     *     8.572052335460611D-01,   8.295657623827684D-01,
     *     7.997278358218391D-01,   7.677774321048262D-01,
     *     7.337900624532268D-01,   6.978504947933158D-01,
     *     6.600610641266270D-01,   6.205261829892429D-01,
     *     5.793452358263617D-01,   5.366241481420199D-01/
C
      DATA XGK(21),XGK(22),XGK(23),XGK(24),XGK(25),XGK(26),XGK(27),
     *     XGK(28),XGK(29),XGK(30),XGK(31)/
     *     4.924804678617786D-01,   4.470337695380892D-01,
     *     4.004012548303944D-01,   3.527047255308781D-01,
     *     3.040732022736251D-01,   2.546369261678898D-01,
     *     2.045251166823099D-01,   1.538699136085835D-01,
     *     1.028069379667370D-01,   5.147184255531770D-02,
     *     0.0D+00              /
C
      DATA WGK(1),WGK(2),WGK(3),WGK(4),WGK(5),WGK(6),WGK(7),WGK(8),
     *     WGK(9),WGK(10)/
     *     1.389013698677008D-03,   3.890461127099884D-03,
     *     6.630703915931292D-03,   9.273279659517763D-03,
     *     1.182301525349634D-02,   1.436972950704580D-02,
     *     1.692088918905327D-02,   1.941414119394238D-02,
     *     2.182803582160919D-02,   2.419116207808060D-02/
C
      DATA WGK(11),WGK(12),WGK(13),WGK(14),WGK(15),WGK(16),WGK(17),
     *     WGK(18),WGK(19),WGK(20)/
     *     2.650995488233310D-02,   2.875404876504129D-02,
     *     3.090725756238776D-02,   3.298144705748373D-02,
     *     3.497933802806002D-02,   3.688236465182123D-02,
     *     3.867894562472759D-02,   4.037453895153596D-02,
     *     4.196981021516425D-02,   4.345253970135607D-02/
C
      DATA WGK(21),WGK(22),WGK(23),WGK(24),WGK(25),WGK(26),WGK(27),
     *     WGK(28),WGK(29),WGK(30),WGK(31)/
     *     4.481480013316266D-02,   4.605923827100699D-02,
     *     4.718554656929915D-02,   4.818586175708713D-02,
     *     4.905543455502978D-02,   4.979568342707421D-02,
     *     5.040592140278235D-02,   5.088179589874961D-02,
     *     5.122154784925877D-02,   5.142612853745903D-02,
     *     5.149472942945157D-02/
C
      DATA WG(1),WG(2),WG(3),WG(4),WG(5),WG(6),WG(7),WG(8)/
     *     7.968192496166606D-03,   1.846646831109096D-02,
     *     2.878470788332337D-02,   3.879919256962705D-02,
     *     4.840267283059405D-02,   5.749315621761907D-02,
     *     6.597422988218050D-02,   7.375597473770521D-02/
C
      DATA WG(9),WG(10),WG(11),WG(12),WG(13),WG(14),WG(15)/
     *     8.075589522942022D-02,   8.689978720108298D-02,
     *     9.212252223778613D-02,   9.636873717464426D-02,
     *     9.959342058679527D-02,   1.017623897484055D-01,
     *     1.028526528935588D-01/
C
C          LIST OF MAJOR VARIABLES
C          -----------------------
C
C          CENTR   - MID POINT OF THE INTERVAL
C          HLGTH   - HALF-LENGTH OF THE INTERVAL
C          ABSC    - ABSCISSA
C          FVAL*   - FUNCTION VALUE
C          RESG    - RESULT OF THE 30-POINT GAUSS RULE
C          RESK    - RESULT OF THE 61-POINT KRONROD RULE
C          RESKH   - APPROXIMATION TO THE MEAN VALUE OF F
C                    OVER (A,B), I.E. TO I/(B-A)
C
C          MACHINE DEPENDENT CONSTANTS
C          ---------------------------
C
C          EPMACH IS THE LARGEST RELATIVE SPACING.
C          UFLOW IS THE SMALLEST POSITIVE MAGNITUDE.
C          OFLOW IS THE LARGEST MAGNITUDE.
C
C
C***FIRST EXECUTABLE STATEMENT
      CALL QMACO(EPMACH,UFLOW,OFLOW)
      CENTR = 5.0D-01*(B+A)
      HLGTH = 5.0D-01*(B-A)
      DHLGTH = DABS(HLGTH)
C
C          COMPUTE THE 61-POINT KRONROD APPROXIMATION TO THE INTEGRAL,
C          AND ESTIMATE THE ABSOLUT ERROR.
C
      RESG = 0.0D+00
      FC = F(CENTR)
      RESK = WGK(31)*FC
      RESABS = DABS(RESK)
      DO 10  J = 1,15
        JTW = J*2
        ABSC = HLGTH*XGK(JTW)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTW) = FVAL1
        FV2(JTW) = FVAL2
        FSUM = FVAL1+FVAL2
        RESG = RESG+WG(J)*FSUM
        RESK = RESK+WGK(JTW)*FSUM
        RESABS = RESABS+WGK(JTW)*(DABS(FVAL1)+DABS(FVAL2))
   10 CONTINUE
      DO 15  J = 1,15
        JTWM1 = J*2-1
        ABSC = HLGTH*XGK(JTWM1)
        FVAL1 = F(CENTR-ABSC)
        FVAL2 = F(CENTR+ABSC)
        FV1(JTWM1) = FVAL1
        FV2(JTWM1) = FVAL2
        FSUM = FVAL1+FVAL2
        RESK = RESK+WGK(JTWM1)*FSUM
        RESABS = RESABS+WGK(JTWM1)*(DABS(FVAL1)+DABS(FVAL2))
   15 CONTINUE
      RESKH = RESK*5.0D-01
      RESASC = WGK(31)*DABS(FC-RESKH)
      DO 20  J = 1,30
        RESASC = RESASC+WGK(J)*(DABS(FV1(J)-RESKH)+DABS(FV2(J)-RESKH))
   20 CONTINUE
      RESULT = RESK*HLGTH
      RESABS = RESABS*DHLGTH
      RESASC = RESASC*DHLGTH
      ABSERR = DABS((RESK-RESG)*HLGTH)
      IF(RESASC.NE.0.0D+00.AND.ABSERR.NE.0.0D+00)
     *  ABSERR = RESASC*DMIN1(1.0D+00,(2.0D+02*ABSERR/RESASC)**1.5D+00)
      IF(RESABS.GT.UFLOW/(5.0D+01*EPMACH)) ABSERR = DMAX1
     *  ((EPMACH*5.0D+01)*RESABS,ABSERR)
      RETURN
      END
