$debug
        SUBROUTINE QNG(F,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
C
C......................................................................
C
C 1.      QNG
C         NON-ADAPTIVE INTEGRATION
C            STANDARD FORTRAN SUBROUTINE
C
C
C 2.       PURPOSE
C             THE ROUTINE CALCULATES AN APPROXIMATION RESULT TO
C             A GIVEN DEFINITE INTEGRAL I = INTEGRAL OF F OVER (A,B),
C             HOPEFULLY SATISFYING FOLLOWING CLAIM FOR ACCURACY
C             ABS(I-RESULT).LE.MAX(EPSABS,EPSREL*ABS(I)).
C
C
C 3.       CALLING SEQUENCE
C             CALL QNG(F,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
C
C          PARAMETERS
C           ON ENTRY
C             F      - REAL
C                      FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                      FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO BE
C                      DECLARED E X T E R N A L  IN THE DRIVE PROGRAM.
C
C             A      - REAL
C                      LOWER LIMIT OF INTEGRATION
C
C             B      - REAL
C                      UPPER LIMIT OF INTEGRATION
C             EPSABS - REAL
C                      ABSOLUTE ACCURACY REQUESTED
C             EPSREL - REAL
C                      RELATIVE ACCURACY REQUESTED
C                      IF EPSABS.LT.0 AND EPSREL.LT.0,
C                      THE ROUTINE WILL END WITH IER = 6.
C
C             ON RETURN
C             RESULT - REAL
C                      APPROXIMATION TO THE INTEGRAL I
C                      RESULT IS OBTAINED BY APPLYING THE 21-POINT
C                      GAUSS-KRONROD RULE (RES21) OBTAINED BY OPTIMAL
C                      ADDITION OF ABSCISSAE TO THE 10-POINT GAUSS RULE
C                      (RES10), OR BY APPLYING THE 43-POINT RULE (RES43)
C                      OBTAINED BY OPTIMAL ADDITION OF ABSCISSAE TO THE
C                      21-POINT GAUSS-KRONROD RULE, OR BY APPLYING THE
C                      87-POINT RULE (RES87) OBTAINED BY OPTIMAL ADDITION
C                      OF ABSCISSAE TO THE 43-POINT RULE.
C
C             ABSERR - REAL
C                      ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                      WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C             NEVAL  - INTEGER
C                      NUMBER OF INTEGRAND EVALUATIONS
C
C             IER    - IER = 0 NORMAL AND RELIABLE TERMINATION OF THE
C                              ROUTINE. IT IS ASSUMED THAT THE REQUESTED
C                              ACCURACY HAS BEEN ACHIEVED.
C                      IER.GT.0 ABNORMAL TERMINATION OF THE ROUTINE. IT IS
C                              ASSUMED THAT THE REQUESTED ACCURACY HAS
C                              NOT BEEN ACHIEVED.
C                      IER = 1 THE MAXIMUM NUMBER OF STEPS HAS BEEN
C                              EXECUTED. THE INTEGRAL IS PROBABY TOO
C                              DIFFICULT TO BE CALCULATED BY QNG.
C                          = 6 THE INPUT IS INVALID, BECAUSE
C                              EPSABS.LT.0 AND EPSREL.LT.0,
C                              RESULT, ABSERR AND NEVAL ARE SET TO ZERO.
C
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C               - F (USER PROVIDED FUNCTION)
C               - QMACO
C               - FORTRAN ABS, AMAX1, AMIN1
C
C........1.........2.........3.........4.........5.........6.........7........8
C
      REAL*8 A,ABSC,ABSERR,B,CENTR,DHLGTH,EPMACH,EPSABS,EPSREL,F,FCENTR,
     * FVAL,FVAL1,FVAL2,FV1,FV2,FV3,FV4,HLGTH,OFLOW,RESULT,RES10,RES21,
     * RES43,RES87,RESABS,RESASC,RESKH,SAVFUN,UFLOW,W10,W21A,W21B,W43A,
     * W43B,W87A,W87B,X1,X2,X3,X4
     	INTEGER IER,IPX,K,L,NEVAL
 	DIMENSION FV1(5),FV2(5),FV3(5),FV4(5),X1(5),X2(5),X3(11),X4(22),
     * W10(5),W21A(5),W21B(6),W43A(10),W43B(12),W87A(21),W87B(23),
     * SAVFUN(21)
C
C	THE FOLLOWING DATA STATEMENTS CONTAIN THE ABSCISSAE
C	AND WEIGHTS OF THE INTEGRATION RULES USED.
C
C	X1	ABSCISSAE COMMON TO THE 10-, 21-, 43- AND 87-POINT
C     		RULE
C	X2	ABSCISSAE COMMON TO THE 21-, 43- AND 87-POINT RULE
C       X3	ABSCISSAE COMMON TO THE 43- AND 87-POINT RULE
C	X4	ABSCISSAE OF THE 87-POINT RULE
C
C	W10	WEIGHTS OF THE 10-POINT FORMULA
C	W21A	WEIGHTS OF THE 21-POINT FORMULA FOR ABSCISSAE X1
C	W21B	WEIGHTS OF THE 21-POINT FORMULA FOR ABSCISSAE X2
C	W43A	WEIGHTS OF THE 43-POINT FORMULA FOR ABSCISSAE X1, X3
C	W43B	WEIGHTS OF THE 43-POINT FORMULA FOR ABSCISSAE X3
C	W87A	WEIGHTS OF THE 87-POINT FORMULA FOR ABSCISSAE X1,
C		X2 AND X3
C	W87B	WEIGHTS OF THE 87-POINT FORMULA FOR ABSCISSAE X4
C
 	DATA X1 /
     *   9.739065285171717D-01,     8.650633666889845D-01,
     *   6.794095682990244D-01,     4.333953941292472D-01,
     *   1.488743389816312D-01 /
        DATA X2 /
     *   9.956571630258081D-01,     9.301574913557082D-01,
     *   7.808177265864169D-01,     5.627571346686047D-01,
     *   2.943928627014602D-01 /
        DATA X3 /
     *   9.993333609019321D-01,     9.874334029080889D-01,
     *   9.548079348142663D-01,     9.001486957483283D-01,
     *   8.251983149831142D-01,     7.321483889893050D-01,
     *   6.228479705377252D-01,     4.994795740710656D-01,
     *   3.649016613465808D-01,     2.222549197766013D-01,
     *   7.465061746138332D-02 /
        DATA X4 /                   9.999029772627292D-01,
     *   9.979898959866787D-01,     9.921754978606872D-01,
     *   9.813581635727128D-01,     9.650576238583846D-01,
     *   9.431676131336706D-01,     9.158064146855072D-01,
     *   8.832216577713165D-01,     8.457107484624157D-01,
     *   8.035576580352310D-01,     7.570057306854956D-01,
     *   7.062732097873218D-01,     6.515894665011779D-01,
     *   5.932233740579611D-01,     5.314936059708319D-01,
     *   4.667636230420228D-01,     3.994248478592188D-01,
     *   3.298748771061883D-01,     2.585035592021616D-01,
     *   1.856953965683467D-01,     1.118422131799075D-01,
     *   3.735212339461987D-02 /
        DATA W10 /
     *   6.667134430868814D-02,     1.494513491505806D-01,
     *   2.190863625159820D-01,     2.692667193099964D-01,
     *   2.955242247147529D-01 /
        DATA W21A /
     *   3.255816230796473D-02,     7.503967481091995D-02,
     *   1.093871588022976D-01,     1.347092173114733D-01,
     *   1.477391049013385D-01 /
        DATA W21B /
     *   1.169463886737187D-02,     5.475589657435200D-02,
     *   9.312545458369761D-02,     1.234919762620659D-01,
     *   1.427759385770601D-01,     1.494455540029169D-01 /
        DATA W43A /                 1.629673428966656D-02,
     *   3.752287612086950D-02,     5.469490205825544D-02,
     *   6.735541460947809D-02,     7.387019963239395D-02,
     *   5.768556059769796D-03,     2.737189059324884D-02,
     *   4.656082691042883D-02,     6.174499520144256D-02,
     *   7.138726726869340D-02 /
        DATA W43B /
     *   1.844477640212414D-03,     1.079868958589165D-02,
     *   2.189536386779543D-02,     3.259746397534569D-02,
     *   4.216313793519181D-02,     5.074193960018458D-02,
     *   5.837939554261925D-02,     6.474640495144589D-02,
     *   6.956619791235648D-02,     7.282444147183321D-02,
     *   7.450775101417512D-02,     7.472214751740301D-02 /
        DATA W87A /
     *   8.148377384149173D-03,     1.876143820156282D-02,
     *   2.734745105005229D-02,     3.367770731163793D-02,
     *   3.693509982042791D-02,     2.884872430211531D-03,
     *   1.368594602271270D-02,     2.328041350288831E-02,
     *   3.087249761171336D-02,     3.569363363941877D-02,
     *   9.152833452022414D-04,     5.399280219300471D-03,
     *   1.094767960111893D-02,     1.629873169678734D-02,
     *   2.108156888920384D-02,     2.537096976925383D-02,
     *   2.918969775647575D-02,     3.237320246720279D-02,
     *   3.478309895036514D-02,     3.641222073135179D-02,
     *   3.725387550304771D-02 /
        DATA W87B /
     *                              2.741455637620724D-04,
     *   1.807124155057943D-03,     4.096869282759165D-03,
     *   6.758290051847379D-03,     9.549957672201647D-03,
     *   1.232944765224485D-02,     1.501044734638895D-02,
     *   1.754896798624319D-02,     1.993803778644089D-02,
     *   2.219493596101229D-02,     2.433914712600081D-02,
     *   2.637450541483921D-02,     2.828691078877120D-02,
     *   3.005258112809270D-02,     3.164675137143993D-02,
     *   3.305041341997850D-02,     3.425509970422606D-02,
     *   3.526241266015668D-02,     3.607698962288870D-02,
     *   3.669860449845609D-02,     3.712054926983258D-02,
     *   3.733422875193504D-02,     3.736107376267902D-02 /
C
C	LIST OF MAJOR VARIABLES
C       -----------------------
C
C	CENTR   - MID POINT OF THE INTEGRATION INTERVAL
C	HLGTH   - HALF-LENGTH OF THE INTEGRATION INTERVAL
C	FCENTR  - FUNCTION VALUE AT MID POINT
C	ABSC    - ABSCISSA
C	FVAL    - FUNCTION VALUE
C	SAVFUN  - ARRAY OF FUNCTION VALUES WHICH HAVE ALREADY
C		  BEEN COMPUTED
C	RES10   - 10-POINT GAUSS RESULT
C	RES21   - 21-POINT KRONROD RESULT
C	RES43   - 43-POINT RESULT
C	RES87   - 87-POINT RESULT
C	RESABS  - APPROXIMATION TO THE INTEGRAL OF ABS(F)
C	RESASC  - APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C
C	MACHINE DEPENDENT CONSTANTS
C	---------------------------
C
C	EPMACH IS THE LARGEST RELATIVE SPACING
C	UFLOW IS THE SMALLEST POSITIVE MAGNITUDE
C	OFLOW IS THE LARGEST MAGNITUDE
C
C***FIRST EXECUTIVE STATEMENT
	CALL QMACO(EPMACH,UFLOW,OFLOW)
C
C	TEST ON VALIDITY OF PARAMETERS
C       ------------------------------
C
 	RESULT = 0.0D+00
        ABSERR = 0.0D+00
        NEVAL = 0
        IER = 6
        IF(EPSABS.LT.0.D0 .AND. EPSREL.LT.0.D0) GO TO 999
        HLGTH = 5.D-01*(B-A)
        DHLGTH = DABS(HLGTH)
        CENTR = 5.D-01*(B+A)
        FCENTR = F(CENTR)
        NEVAL = 21
        IER = 1
C
C	COMPUTE THE INTEGRAL USING THE 10- AND 21-POINT FORMULA.
C
	DO 70 L = 1,3
        GO TO (5,25,45),L
   5	RES10 = 0.0D+00
   	RES21 = W21B(6)*FCENTR
        RESABS = W21B(6)*DABS(FCENTR)
        DO 10 K=1,5
          ABSC = HLGTH*X1(K)
          FVAL1 = F(CENTR+ABSC)
          FVAL2 = F(CENTR-ABSC)
          FVAL = FVAL1+FVAL2
          RES10 = RES10+W10(K)*FVAL
          RES21 = RES21+W21A(K)*FVAL
          RESABS = RESABS+W21A(K)*(DABS(FVAL1)+DABS(FVAL2))
          SAVFUN(K) = FVAL
          FV1(K) = FVAL1
          FV2(K) = FVAL2
   10 	CONTINUE
   	IPX = 5
        DO 15 K=1,5
          IPX = IPX+1
          ABSC = HLGTH*X2(K)
          FVAL1 = F(CENTR+ABSC)
          FVAL2 = F(CENTR-ABSC)
          FVAL = FVAL1+FVAL2
          RES21 = RES21+W21B(K)*FVAL
          RESABS = RESABS+W21B(K)*(DABS(FVAL1)+DABS(FVAL2))
          SAVFUN(IPX) = FVAL
          FV3(K) = FVAL1
          FV4(K) = FVAL2
   15	CONTINUE
C
C		TEST FOR CONVERGENCE
C
	RESULT = RES21*HLGTH
        RESABS = RESABS*DHLGTH
        RESKH = 5.0D-01*RES21
        RESASC = W21B(6)*DABS(FCENTR-RESKH)
      DO 20 K = 1,5
        RESASC = RESASC+W21A(K)*(DABS(FV1(K)-RESKH)+DABS(FV2(K)-RESKH))
     *                 +W21B(K)*(DABS(FV3(K)-RESKH)+DABS(FV4(K)-RESKH))
  20  CONTINUE
  	ABSERR = DABS((RES21-RES10)*HLGTH)
        RESASC = RESASC*DHLGTH
        GO TO 65
C
C	COMPUTE THE INTEGRAL USING THE 43-POINT FORMULA
C
   25	RES43 = W43B(12)*FCENTR
   	NEVAL = 43
        DO 30 K=1,10
          RES43 = RES43+SAVFUN(K)*W43A(K)
   30	CONTINUE
   	DO 40 K=1,11
          IPX = IPX+1
          ABSC = HLGTH*X3(K)
          FVAL = F(ABSC+CENTR)+F(CENTR-ABSC)
          RES43 = RES43+FVAL*W43B(K)
          SAVFUN(IPX) = FVAL
   40	CONTINUE
C
C	TEST FOR CONVERGENCE
C
	RESULT = RES43*HLGTH
        ABSERR = DABS((RES43-RES21)*HLGTH)
        GO TO 65
C
C	COMPUTE THE INTEGRAL USING THE 87-POINT FORMULA
C
   45	RES87 = W87B(23)*FCENTR
        NEVAL = 87
        DO 50 K=1,21
          RES87 = RES87+SAVFUN(K)*W87A(K)
   50	CONTINUE
   	DO 60 K=1,22
          ABSC = HLGTH*X4(K)
          RES87 = RES87+W87B(K)*(F(ABSC+CENTR)+F(CENTR-ABSC))
   60	CONTINUE
   	RESULT = RES87*HLGTH
        ABSERR = DABS((RES87-RES43)*HLGTH)
   65	IF(RESASC.NE.0.0D0.AND.ABSERR.NE.0.0D0)
     *  ABSERR = RESASC*DMIN1(1.0D0,(2.D+02*ABSERR/RESASC)**1.5D+00)
     	IF(RESABS.GT.UFLOW/(5.0D+1*EPMACH)) ABSERR = DMAX1
     *    ((EPMACH*5.D+01)*RESABS,ABSERR)
     	IF(ABSERR.LE.DMAX1(EPSABS,EPSREL*DABS(RESULT))) IER = 0
C***JUMP OUT OF DO-LOOP
	IF (IER.EQ.0) GO TO 999
   70	CONTINUE
  999	RETURN
   	END
